@@ -1,4 +1,4 @@
-/* $Id: file_conduit.c,v 1.9 1999/12/09 03:35:09 eskil Exp $ */
+/* $Id: file_conduit.c,v 1.12 2000/02/27 12:03:57 eskil Exp $ */
 
 #include <glib.h>
 #include <gnome.h>
@@ -36,8 +36,12 @@
 #define MAXDBS 256
 /* pilot stuff end */
 
+GnomePilotConduit *conduit_get_gpilot_conduit (guint32 pilotId);
+void conduit_destroy_gpilot_conduit (GnomePilotConduit *conduit);
+
 typedef struct file_db {
         gchar name[256];
+        gchar dbname[256];
         gint flags;
         gulong creator;
         gulong type;
@@ -49,6 +53,7 @@ gnome_real_pilot_conduit_file_install_db (GnomePilotConduitFile *file_conduit,
 					  int psock,
 					  gchar *src_file)
 {
+	GnomePilotDBInfo info;
 	struct pi_file *file;
 #ifdef DEBUG
 	g_print ("in gnome_real_pilot_conduit_file_install_db\nsrc_file is %s\n", src_file);
@@ -73,10 +78,19 @@ gnome_real_pilot_conduit_file_install_db (GnomePilotConduitFile *file_conduit,
 	fprintf(stderr,"file_conduit: Installing %s... ", src_file);
 	fflush(stdout);
 #endif
+	pi_file_get_info (file, (PI_DBINFO (&info)));
+	g_message(_("Installing %s from %s..."),PI_DBINFO (&info)->name, src_file);
+	gnome_pilot_conduit_message(GNOME_PILOT_CONDUIT(file_conduit),
+				    _("Installing %s from %s..."),
+				    PI_DBINFO (&info)->name,
+				    src_file);
 	if (pi_file_install (file, psock, 0) < 0) {
 #ifdef DEBUG
 		fprintf(stderr,"failed.\n");
 #endif
+		gnome_pilot_conduit_error(GNOME_PILOT_CONDUIT(file_conduit),
+					  _("Installing %s failed"),
+					  PI_DBINFO (&info)->name);
 		pi_file_close (file);
 		return -1;
 	} else {
@@ -89,8 +103,8 @@ gnome_real_pilot_conduit_file_install_db (GnomePilotConduitFile *file_conduit,
 	}
 };
 
-int compare(file_db *d1,
-	    file_db *d2)
+static int 
+compare(file_db *d1, file_db *d2)
 {
 	/* types of 'appl' sort later then other types */
 	if(d1->creator == d2->creator) {
@@ -112,7 +126,7 @@ gnome_real_pilot_conduit_file_restore_directory (GnomePilotConduitFile *file_con
 	DIR *dir;
 	struct dirent *dirent;
 	GnomePilotDBInfo info;
-	file_db *db[MAXDBS];
+	file_db **db;
 	int dbcount = 0;
 	int i, j, max, size;
 	struct pi_file *file;
@@ -121,6 +135,7 @@ gnome_real_pilot_conduit_file_restore_directory (GnomePilotConduitFile *file_con
 
 	dir = opendir (src_dir);
 
+	db = (file_db**)g_malloc (MAXDBS * sizeof (file_db*));
 	while( (dirent = readdir(dir)) ) {
 
 		/* skip dotfiles. */
@@ -128,7 +143,6 @@ gnome_real_pilot_conduit_file_restore_directory (GnomePilotConduitFile *file_con
 			continue;
 
 		db[dbcount] = (file_db*)g_malloc (sizeof (file_db));
-
 		g_snprintf (db[dbcount]->name, 255, "%s/%s", src_dir, dirent->d_name);
 
 		file = pi_file_open (db[dbcount]->name);
@@ -138,10 +152,12 @@ gnome_real_pilot_conduit_file_restore_directory (GnomePilotConduitFile *file_con
 #endif
 			/* Skip this file */
 			continue;
+
 		}
 
 		pi_file_get_info (file, (PI_DBINFO (&info)));
 
+		g_snprintf (db[dbcount]->dbname, 255, "%s", PI_DBINFO (&info)->name);
 		db[dbcount]->creator = PI_DBINFO (&info)->creator;
 		db[dbcount]->type = PI_DBINFO (&info)->type;
 		db[dbcount]->flags = PI_DBINFO (&info)->flags;
@@ -193,11 +209,19 @@ gnome_real_pilot_conduit_file_restore_directory (GnomePilotConduitFile *file_con
 		fprintf (stdout,"file_conduit: Restoring %s... ", db[i]->name);
 		fflush (stdout);
 #endif
-		g_message("Restoring %s... ", db[i]->name);
+		g_message(_("Restoring %s from %s... "), db[i]->dbname, db[i]->name);
+		gnome_pilot_conduit_message(GNOME_PILOT_CONDUIT(file_conduit),
+					    _("Restoring %s from %s... "), 
+					    db[i]->dbname, 
+					    db[i]->name);
 		if (pi_file_install(file, psock, 0)<0) {
 #ifdef DEBUG
 			printf("failed.\n");
 #endif
+			g_message(_("Restore of %s failed"),db[i]->dbname);
+			gnome_pilot_conduit_error(GNOME_PILOT_CONDUIT(file_conduit),
+						    _("Restore of %s failed"),
+						    db[i]->dbname);
 		} else {
 #ifdef DEBUG
 			printf("OK\n");
@@ -210,6 +234,11 @@ gnome_real_pilot_conduit_file_restore_directory (GnomePilotConduitFile *file_con
 #ifdef DEBUG
 	fprintf (stderr,"file_conduit: Restore done\n");
 #endif
+
+	for(i=0;i<dbcount;i++){
+		g_free(db[i]);
+	}
+	g_free(db);
 	return 0;
 }
 
@@ -226,6 +255,7 @@ conduit_get_gpilot_conduit (guint32 pilotId)
 
 	return retval;
 }
+
 void
 conduit_destroy_gpilot_conduit (GnomePilotConduit *conduit)
 {
