@@ -1,6 +1,9 @@
-#include <gpilotd/gpilotd-app.h>
+#include "config.h"
+#include <gnome.h>
+#include <gpilotd/gnome-pilot-client.h>
 #include <libgnorba/gnorba.h>
 
+GnomePilotClient *gpc;
 CORBA_Environment ev;
 GSList *handles;
 GSList *failed,*notfailed;
@@ -10,89 +13,37 @@ GtkWidget *dialog;
 
 int now = 0, later = 0;
 char *debug_modules = NULL;
+char *pilot_arg=NULL;
+
 static const struct poptOption options[] = {
 	{"now", 'n', POPT_ARG_NONE, &now, 0, N_("Install immediately"), NULL},
 	{"later", 'l', POPT_ARG_NONE, &later, 0, N_("Install delayed"), NULL},
+	{"pilot", 'p', POPT_ARG_STRING, &pilot_arg, 0, N_("Pilot to install to"), N_("PILOT")},
 	{NULL, '\0', 0, NULL, 0} /* end the list */
 };
 
-void 
-gpilotd_connect_cb(const gchar *id,const GNOME_Pilot_PilotUser *user) 
+static void 
+gpilotd_request_completed(GnomePilotClient *gpc, gchar *pilot_id, gint handle, gpointer data)
 {
-}
-
-void 
-gpilotd_disconnect_cb(const gchar *id) 
-{
-}
-
-void 
-gpilotd_request_completed(const gchar *id,unsigned long handle) 
-{
-	g_message("%s completed %d",id,handle);
+	g_message("%s completed %d",pilot_id,handle);
 	handles = g_slist_remove(handles,(gpointer)handle);
 	if (handles == NULL)
 		gnome_dialog_close(GNOME_DIALOG(dialog));
 }
 
-void 
-gpilotd_userinfo_requested(const gchar *device,const  GNOME_Pilot_PilotUser *user) 
-{
-}
-
-void 
-gpilotd_sysinfo_requested(const gchar *id,const  GNOME_Pilot_SysInfo *info) 
-{
-}
-
-void 
-gpilotd_conduit_start(const gchar *id, 
-		      const gchar *conduit, 
-		      const gchar *db_name) { 
-	g_message("conduit_start(%s,%s,%s)",id,conduit,db_name);
-}
-
-void 
-gpilotd_conduit_end(const gchar *id,
-		    const gchar *conduit) { 
-	g_message("conduit_end(%s)",id);
-}
-
-void 
-gpilotd_conduit_progress(const gchar *id, 
-			 const gchar *conduit, 
-			 long current, 
-			 long total) {
-	g_message("conduit_progress(%s,%s,%d/%d)",id, conduit, current, total);
-}
-
-void 
-gpilotd_conduit_message(const gchar *id, 
-			const gchar *conduit, 
-			const gchar *message) {
-	g_message("conduit_message(%s,%s,%s)",id,conduit,message);
-}
-
-void 
-gpilotd_conduit_error(const gchar *id, 
-		      const gchar *conduit, 
-		      const gchar *message) {
-	g_warning("conduit_error(%s,%s,%s)",id,conduit,message);
-}
-
 static void 
 cancel_install(GnomeDialog *w,GSList *list) 
 {
 	GSList *e;
 
 	for(e=list;e;e = g_slist_next(e)) {
-		gpilotd_remove_request((gint)e->data);  
+		gnome_pilot_client_remove_request(gpc,(gint)e->data);  
 	}
 	g_slist_free(list);
 
 }
 
-void 
+static void 
 show_warning_dialog(gchar *mesg,...) 
 {
 	char *tmp;
@@ -113,9 +64,12 @@ main(int argc,char *argv[])
 	int err,i;
 	GNOME_Pilot_Survival survive;
         poptContext pctx;
-	gchar **pilots = NULL;
-        char **args;
+	GList *pilots = NULL;
+        const char **args;
         
+	bindtextdomain (PACKAGE, GNOMELOCALEDIR);
+	textdomain (PACKAGE);
+
 	if(argc<2) {
 		g_message("usage : %s [--now|--later] [FILE]",argv[0]);
 		exit(1);
@@ -126,25 +80,21 @@ main(int argc,char *argv[])
 					 options,0,&pctx,
 					 0,&ev);
 
-	/* 1. initialize the gpilotd connection */
-	if (gpilotd_init(&argc,argv) != 0) {
-		g_warning(_("Cannot start the GnomePilot daemon"));
-		show_warning_dialog(_("Cannot start the GnomePilot daemon"));
-		exit(1);
-	}
-	/* 1.1 connect to gpilotd */
-	if (gpilotd_connect() != 0)  {
-		g_warning(_("Cannot connect to the GnomePilot daemon"));
-		show_warning_dialog(_("Cannot connect to the GnomePilot daemon"));
-		exit(1);
-	}
 	
-	err = gpilotd_get_pilot_names (&pilots);
-	if (err != 0 || pilots == NULL) {
-                g_warning (_("Unable to get pilot names"));
-		show_warning_dialog(_("Unable to get pilot names"));
-		exit(1);
-        }
+	gpc = GNOME_PILOT_CLIENT(gnome_pilot_client_new());
+	gtk_signal_connect(GTK_OBJECT(gpc),"completed_request", (GtkSignalFunc)gpilotd_request_completed, NULL);
+	gnome_pilot_client_connect_to_daemon(gpc);
+
+	if (pilot_arg!=NULL) {
+		pilots = g_list_append(pilots,g_strdup(pilot_arg));
+	} else {
+		err = gnome_pilot_client_get_pilots (gpc,&pilots);
+		if (err !=GPILOTD_OK || pilots == NULL) {
+			g_warning (_("Unable to get pilot names"));
+			show_warning_dialog(_("Unable to get pilot names"));
+			exit(1);
+		}
+	}
 
 	notfailed = failed = handles = NULL;
 
@@ -157,8 +107,11 @@ main(int argc,char *argv[])
 
 	while(args && args[i]!=NULL) {
 		if (g_file_exists(args[i]) != 0) {
-			handle = gpilotd_install_file(pilots[0],args[i],survive,0);
-			if (handle > 0) {
+			gint err;
+			err = gnome_pilot_client_install_file(gpc,
+							      pilots->data, /* get first pilot */
+							      args[i],survive,0,&handle);
+			if (err == GPILOTD_OK) {
 				handles = g_slist_prepend(handles,(gpointer)handle);
 				notfailed = g_slist_prepend(notfailed,args[i]);
 			} else {
@@ -190,10 +143,10 @@ main(int argc,char *argv[])
 		{
 			GSList *e;
 			if (message == NULL)
-				message = g_strdup(_("Installing :\n"));
+				message = g_strdup_printf(_("Installing to %s:\n"),pilots[0]);
 			else {
 				gchar *tmp;
-				tmp = g_strconcat(message,"\nInstalling :\n",NULL);
+				tmp = g_strconcat(message,"\nInstalling to ",pilots[0],":\n",NULL);
 				g_free(message);
 				message = tmp;
 			}
@@ -233,9 +186,9 @@ main(int argc,char *argv[])
 		g_free(message);
 	}
 
-	gpilotd_shutdown(); 
+	/* gpilotd_shutdown(); */
 	CORBA_ORB_shutdown(gnome_CORBA_ORB(), CORBA_FALSE, &ev);
 
-  return 0;
+	return 0;
 }
 
