@@ -14,12 +14,70 @@
 #include <pwd.h>
 #include <signal.h>
 #include <errno.h>
+#include <ctype.h>
 #include <gpilotd/gnome-pilot-conduit.h>
 #include <gpilotd/gnome-pilot-conduit-standard-abs.h>
 
 #include "memo_file_conduit.h"
 
-#define CONDUIT_VERSION "0.6"
+#define MC_DEBUG
+
+#ifdef MC_DEBUG
+#define LOG(args...) g_log (G_LOG_DOMAIN, \
+                            G_LOG_LEVEL_MESSAGE, \
+                             "memo_file: "##args)
+#else
+#define LOG(args...)
+#endif
+
+GnomePilotConduit *conduit_get_gpilot_conduit (guint32 pilotId);
+void conduit_destroy_gpilot_conduit (GnomePilotConduit *conduit);
+
+#define CONDUIT_VERSION "0.7"
+
+static void 
+load_configuration(ConduitCfg **c,guint32 pilotId) 
+{
+	gchar prefix[256];
+	gchar *buf;
+	g_return_if_fail(c!=NULL);
+	
+	g_snprintf(prefix,255,CONFIG_PREFIX,pilotId);
+ 
+	*c = g_new0(ConduitCfg,1);
+	gnome_config_push_prefix(prefix);
+	(*c)->sync_type = GnomePilotConduitSyncTypeCustom; /* this will be reset by capplet */
+	(*c)->open_secret = gnome_config_get_bool("open secret=FALSE"); 
+	
+	buf = gnome_config_get_string("file mode=0600");
+	(*c)->file_mode =(mode_t)strtol(buf,NULL,0);
+	g_free(buf);
+
+	buf = gnome_config_get_string("dir mode=0700");
+	(*c)->dir_mode =(mode_t)strtol(buf,NULL,0);
+	g_free(buf);
+
+	buf = gnome_config_get_string("secret mode=0600");
+	(*c)->secret_mode =(mode_t)strtol(buf,NULL,0);
+	g_free(buf);
+  
+	(*c)->dir = gnome_config_get_string("dir");
+	(*c)->ignore_end=gnome_config_get_string("ignore end");
+	(*c)->ignore_start=gnome_config_get_string("ignore start");
+	gnome_config_pop_prefix();
+	(*c)->pilotId = pilotId;
+}
+
+static void 
+destroy_configuration(ConduitCfg **c) 
+{
+	g_return_if_fail(c!=NULL);
+	if((*c)->dir) g_free((*c)->dir);
+	if((*c)->ignore_start) g_free((*c)->ignore_start);
+	if((*c)->ignore_end) g_free((*c)->ignore_end);
+	g_free(*c);
+	*c = NULL;
+}
 
 static IterateData *
 new_iterate_data(int _flag,int _archived) 
@@ -86,7 +144,10 @@ newfilename(MemoLocalRecord * r)
 	while( rec < end && isspace(*rec) )		/* skip whitespace */
 		++rec;
 	
-	for( i = 0; rec < end; ++i, ++rec) {
+	/* Go only upto 63 characters for file names. That should be enough. 
+	   protect_name() can maximally generate 4*n character name
+	*/
+	for( i = 0; rec < end && i < 63; ++i, ++rec) {
 		if( *rec == '\n' )
 			break;
 		else
@@ -180,7 +241,7 @@ spool_foreach(MemoLocalRecord *local,GnomePilotConduitStandardAbs *abs)
 	if(!local) return;
 	if (local->local.attr==GnomePilotRecordDeleted) return;
 	
-	LOG("spool_foreach",1);
+	LOG("spool_foreach");
 	generate_name(local,abs);
 	
 	if(local->local.secret) mode=GET_CONDUIT_CFG(abs)->secret_mode;
@@ -272,7 +333,8 @@ spool_records(GnomePilotConduitStandardAbs *abs)
 	gchar filename[FILENAME_MAX];
 	gchar entry[128];
 	int f;
-	
+
+	g_return_if_fail(GET_CONDUIT_CFG(abs)->dir != NULL);
 	/* backup, in case we die before all is spooled */
 	backup_directory(abs);
 	g_snprintf(filename,FILENAME_MAX-1,"%s/.categories",GET_CONDUIT_CFG(abs)->dir);
@@ -286,7 +348,6 @@ spool_records(GnomePilotConduitStandardAbs *abs)
 	}
 	close(f);
 
-
 	g_list_foreach(GET_CONDUIT_DATA(abs)->records,(GFunc)spool_foreach,abs);
 
 	nuke_backup(abs);
@@ -360,7 +421,9 @@ iterate_foreach(MemoLocalRecord *local,IterateData *d)
 		if(d->archived != local->local.archived) accept = FALSE;
 	if(d->flag>=0)
 		if(d->flag != local->local.attr) accept = FALSE;
-	
+
+	if(local->ignore == TRUE) accept = FALSE;
+
 	if(accept) { 
 		if(d->prev) 
 			d->prev->next = local;
@@ -384,6 +447,7 @@ create_deleted_record_foreach(gchar *key, LoadInfo *li,GList **records)
 	local->length = 0;
 	local->record = NULL;
 	local->filename = g_strdup(key);
+	local->ignore = FALSE;
 
 	*records = g_list_append(*records,local);
 }
@@ -437,7 +501,7 @@ load_categories(GnomePilotConduitStandardAbs *abs)
 	FILE *f;
 	gchar filename[FILENAME_MAX];
 	
-	LOG("load_categories",1);
+	LOG("load_categories");
 	categories = g_hash_table_new(g_str_hash,g_str_equal);
 	g_snprintf(filename,FILENAME_MAX-1,"%s/.categories",GET_CONDUIT_CFG(abs)->dir);
 
@@ -502,16 +566,16 @@ load_records(GnomePilotConduitStandardAbs *abs)
 	MemoLocalRecord *local;
 	GHashTable *categories;
 	int category;
+	int total=0,updated=0,deleted=0,newrecs=0;
 	
-	LOG("load_records",1);
+	LOG("load_records");
 
-	
 	if((dir=opendir(GET_CONDUIT_CFG(abs)->dir))==NULL) {
 		LOG("load_records cannot open %s",GET_CONDUIT_CFG(abs)->dir);
 		return;
 	}
 	if((categories = load_categories(abs))==NULL) {
-		LOG("no categories, no records",1);
+		LOG("no categories, no records");
 		closedir(dir);
 		return;
 	}
@@ -560,11 +624,10 @@ load_records(GnomePilotConduitStandardAbs *abs)
 						LOG("Ignoring %s",l_de->d_name);
 						continue;
 					}
-					LOG("Found local file %s",l_de->d_name);
 					
-					local = (MemoLocalRecord*)g_malloc(sizeof(MemoLocalRecord));
-					local->filename = (gchar*)g_malloc(FILENAME_MAX);
-					g_snprintf(local->filename,FILENAME_MAX,"%s/%s",filename,l_de->d_name);
+					local = g_new0(MemoLocalRecord,1);
+					local->filename = g_strdup_printf("%s/%s",filename,l_de->d_name);
+					
 					if((info=g_hash_table_lookup(loadinfo,local->filename))!=NULL) {
 						local->local.ID = info->id;
 						local->local.secret = info->secret;
@@ -578,14 +641,24 @@ load_records(GnomePilotConduitStandardAbs *abs)
 					}
 					local->local.archived = 0;
 					local->category = category;
+					local->ignore = FALSE;
 					load_record(abs,local);
 					
 					GET_CONDUIT_DATA(abs)->records = g_list_append(GET_CONDUIT_DATA(abs)->records,local);
+					total++;
+					switch(local->local.attr) {
+					case GnomePilotRecordDeleted: deleted++; break;
+					case GnomePilotRecordModified: updated++;break;
+					case GnomePilotRecordNew: newrecs++; break;
+					default: break;
+					}
+					LOG("Found local file %s, state %d",l_de->d_name,local->local.attr);
 				}
 				closedir(l_dir);      
 			}
 		}
 		if(g_hash_table_size(loadinfo)>0) {
+			deleted += g_hash_table_size(loadinfo);
 			g_hash_table_foreach(loadinfo,(GHFunc)create_deleted_record_foreach,&GET_CONDUIT_DATA(abs)->records);
 			g_hash_table_foreach(loadinfo,(GHFunc)free_loadinfo_foreach,NULL);
 		}
@@ -595,33 +668,42 @@ load_records(GnomePilotConduitStandardAbs *abs)
 	g_hash_table_foreach(categories,(GHFunc)free_str_foreach,NULL);
 	g_hash_table_destroy(categories);
 		
-	LOG("records = %d",g_list_length(GET_CONDUIT_DATA(abs)->records));
+	gnome_pilot_conduit_standard_abs_set_num_local_records(abs,total);
+	gnome_pilot_conduit_standard_abs_set_num_updated_local_records(abs,updated);
+	gnome_pilot_conduit_standard_abs_set_num_new_local_records(abs,newrecs);
+	gnome_pilot_conduit_standard_abs_set_num_deleted_local_records(abs,deleted);
+	
+	LOG("records: total = %d updated = %d new = %d deleted = %d",total,updated,newrecs,deleted);
 }
 
 
 static gint 
 pre_sync(GnomePilotConduit *c, GnomePilotDBInfo *dbi) 
 {
-	int retval;
 	int l;
 	unsigned char *buf;
   
 	g_message ("MemoFile Conduit v %s", CONDUIT_VERSION);
 
-	LOG("PreSync",1);
+	LOG("PreSync");
 
 	GET_CONDUIT_DATA(c)->dbi=dbi;
   
-	load_records((GnomePilotConduitStandardAbs*)c); 
 	buf = (unsigned char*)g_malloc(0xffff);
 	if((l=dlp_ReadAppBlock(dbi->pilot_socket,dbi->db_handle,0,(unsigned char *)buf,0xffff))<0) {
-		LOG("dlp_ReadAppBlock(...) failed",1);
+		LOG("dlp_ReadAppBlock(...) failed");
 		g_free(buf);
 		return -1;
 	}
 	unpack_MemoAppInfo(&(GET_CONDUIT_DATA(c)->ai),buf,l);
 	g_free(buf);
 
+	if (GET_CONDUIT_CFG(c)->dir==NULL) {
+		return -1;
+	}
+
+	load_records((GnomePilotConduitStandardAbs*)c); 
+
 	/* If local store is empty force the slow sync. */
 	if(g_list_length(GET_CONDUIT_DATA(c)->records)==0){
 		gnome_pilot_conduit_standard_set_slow((GnomePilotConduitStandard*)c);
@@ -636,7 +718,7 @@ match_record	(GnomePilotConduitStandardAbs *abs,
 		 gpointer data)
 {
 	GList *tmp;
-	LOG("MatchRecord",1);
+	LOG("MatchRecord");
 
 	g_return_val_if_fail(local!=NULL,-1);
 	g_return_val_if_fail(remote!=NULL,-1);
@@ -655,7 +737,7 @@ free_match	(GnomePilotConduitStandardAbs *abs,
 		 MemoLocalRecord **local,
 		 gpointer data)
 {
-	LOG("FreeMatch",1);
+	LOG("FreeMatch");
 
 	g_return_val_if_fail(local!=NULL,-1);
 	g_return_val_if_fail(*local!=NULL,-1);
@@ -669,7 +751,7 @@ archive_local (GnomePilotConduitStandardAbs *abs,
 	       MemoLocalRecord *local,
 	       gpointer data)
 {
-	LOG("ArchiveLocal",1);
+	LOG("ArchiveLocal");
 	g_return_val_if_fail(local!=NULL,-1);
 	local->local.archived = 1; 
 	return 0;
@@ -681,7 +763,7 @@ archive_remote (GnomePilotConduitStandardAbs *abs,
 		PilotRecord *remote,
 		gpointer data)
 {
-	LOG("ArchiveRemote",1);
+	LOG("ArchiveRemote");
 	g_return_val_if_fail(remote!=NULL,-1);
 
 	if(local!=NULL)  local->local.archived = 1; 
@@ -705,7 +787,7 @@ store_remote (GnomePilotConduitStandardAbs *abs,
 	GList *tmp;
 	ConduitData *cd;
 
-	LOG("StoreRemote",1);
+	LOG("StoreRemote");
 	
 	g_return_val_if_fail(remote!=NULL,-1);
 	
@@ -731,6 +813,7 @@ store_remote (GnomePilotConduitStandardAbs *abs,
 	local->local.secret = remote->secret;
 	local->length = remote->length;
 	local->category = remote->category; 
+	local->ignore = FALSE;
 	if(local->length) {
 		/* paranoia check */
 		if (remote->record==NULL) {
@@ -751,7 +834,7 @@ clear_status_archive_local (GnomePilotConduitStandardAbs *abs,
 			    MemoLocalRecord *local,
 			    gpointer data)
 {
-	LOG("ClearStatusArchiveLocal",1);
+	LOG("ClearStatusArchiveLocal");
 	g_return_val_if_fail(local!=NULL,-1);
 	local->local.archived=0;
 	return 0;
@@ -762,7 +845,7 @@ iterate (GnomePilotConduitStandardAbs *abs,
 	 MemoLocalRecord **local,
 	 gpointer data)
 {
-	LOG("Iterate",1);
+	LOG("Iterate");
 	g_return_val_if_fail(local!=NULL,-1);
 	if(!*local) {
 		/* setup the links */
@@ -808,7 +891,7 @@ static gint
 purge (GnomePilotConduitStandardAbs *abs,
        gpointer data)
 {
-	LOG("Purge",1);
+	LOG("Purge");
 
 	g_list_foreach(GET_CONDUIT_DATA(abs)->records,
 		       (GFunc)purge_records_foreach,
@@ -824,9 +907,10 @@ set_status (GnomePilotConduitStandardAbs *abs,
 	    gint status,
 	    gpointer data)
 {
-	LOG("SetStatus",1);
+	LOG("SetStatus %d",status);
 	g_return_val_if_fail(local!=NULL,-1);
 	local->local.attr = status; 
+	if(status==GnomePilotRecordDeleted) local->ignore=TRUE;
 	return 0;
 }
 
@@ -836,7 +920,7 @@ set_archived (GnomePilotConduitStandardAbs *abs,
 	      gint archived,
 	      gpointer data)
 {
-	LOG("SetArchived",1);
+	LOG("SetArchived");
 	g_return_val_if_fail(local!=NULL,-1);
 	local->local.archived = archived; 
 	return 0;
@@ -848,7 +932,7 @@ set_pilot_id (GnomePilotConduitStandardAbs *abs,
 	      guint32 ID,
 	      gpointer data)
 {
-	LOG("SetPilotId, ID = %lu",ID);
+	LOG("SetPilotId, ID = %u",ID);
 	g_return_val_if_fail(local!=NULL,-1);
 	local->local.ID = ID; 
 	return 0;
@@ -860,7 +944,7 @@ compare (GnomePilotConduitStandardAbs *abs,
 	 PilotRecord *remote,
 	 gpointer data)
 {
-	LOG("Compare",1);
+	LOG("Compare");
 	
 	g_return_val_if_fail(local!=NULL,-1);
 	g_return_val_if_fail(remote!=NULL,-1);
@@ -874,7 +958,7 @@ compare_backup (GnomePilotConduitStandardAbs *abs,
 		PilotRecord *remote,
 		gpointer data)
 {
-	LOG("CompareBackup",1);
+	LOG("CompareBackup");
 	g_return_val_if_fail(local!=NULL,-1);
 	g_return_val_if_fail(remote!=NULL,-1);
 	if(local->record==NULL || remote->record == NULL) return -1;
@@ -887,7 +971,7 @@ free_transmit (GnomePilotConduitStandardAbs *abs,
 	       PilotRecord **remote,
 	       gpointer data)
 {
-	LOG("FreeTransmit",1);
+	LOG("FreeTransmit");
 	g_return_val_if_fail(local!=NULL,-1);
 	g_return_val_if_fail(remote!=NULL,-1);
 	g_return_val_if_fail(*remote!=NULL,-1);
@@ -901,7 +985,7 @@ static gint
 delete_all (GnomePilotConduitStandardAbs *abs,
 	    gpointer data)
 {
-	LOG("DeleteAll",1);
+	LOG("DeleteAll");
 	g_list_foreach(GET_CONDUIT_DATA(abs)->records,
 		       (GFunc)delete_records_foreach,
 		       NULL);
@@ -951,7 +1035,7 @@ conduit_get_gpilot_conduit (guint32 pilotId)
 	gnome_pilot_conduit_construct(GNOME_PILOT_CONDUIT(retval),"memo_file");
 
 	
-	LOG("creating memo_file conduit",0);
+	LOG("creating memo_file conduit");
 
 	g_assert (retval != NULL);
 	gtk_signal_connect (retval, "match_record", (GtkSignalFunc) match_record, NULL);
@@ -976,8 +1060,19 @@ conduit_get_gpilot_conduit (guint32 pilotId)
 	load_configuration(&cc,pilotId);
 	gtk_object_set_data(GTK_OBJECT(retval),OBJ_DATA_CONFIG,cc);
 	gtk_object_set_data(GTK_OBJECT(retval),OBJ_DATA_CONDUIT,cd);
+
+	if(cc->dir==NULL) {
+		g_warning(_("No dir specified. Please run memo_file conduit capplet first."));
+		gnome_pilot_conduit_error(GNOME_PILOT_CONDUIT(retval),
+					  _("No dir specified. Please run memo_file conduit capplet first."));
+		/* FIXME: the following is probably the right way to go, 
+		   but with current state it doesn't work. (capplet wouldn't start up)
+                destroy_configuration(&cc);
+		return NULL;
+		*/
+	} 
 	
-	if(cc->open_secret) gnome_pilot_conduit_standard_abs_set_db_open_mode(GNOME_PILOT_CONDUIT(retval),
+	if(cc->open_secret) gnome_pilot_conduit_standard_abs_set_db_open_mode(GNOME_PILOT_CONDUIT_STANDARD_ABS(retval),
 									      dlpOpenReadWrite|dlpOpenSecret);
 	return GNOME_PILOT_CONDUIT (retval);
 }
@@ -987,7 +1082,7 @@ conduit_destroy_gpilot_conduit (GnomePilotConduit *conduit)
 {
 	ConduitData *cd=GET_CONDUIT_DATA(conduit);
 	ConduitCfg  *cfg=GET_CONDUIT_CFG(conduit);
-	LOG("destroying memo_file conduit",1);
+	LOG("destroying memo_file conduit");
 	
 	g_list_foreach(cd->records,(GFunc)free_records_foreach,cd->records);
 	g_list_free(cd->records);
