@@ -422,6 +422,13 @@ gnome_pilot_conduit_standard_real_copy_to_pilot     (GnomePilotConduitStandard *
 		return -1;
 	}
 	
+	/* Set the counters for the progress bar, note, total_records is set
+	   in standard_abs_open_db */
+	if (conduit->num_local_records == -1) {
+		conduit->num_local_records = conduit->total_records;
+	}
+	conduit->total_progress += conduit->num_updated_local_records;
+
 	/* nuke all records */
 	if (dlp_DeleteRecord (dbinfo->pilot_socket, dbinfo->db_handle, 1, 0) < 0) {
 		g_warning(_("Unable to delete all records in pilot database, aborting operation."));
@@ -450,8 +457,6 @@ gnome_pilot_conduit_standard_real_copy_to_pilot     (GnomePilotConduitStandard *
 						 &assigned_id);
 			if (retval>0) gnome_pilot_conduit_standard_abs_set_pilot_id(conduit,local,assigned_id);
 		        gnome_pilot_conduit_standard_abs_free_transmit (conduit, local, &remote);
-			/* HEST */
-			/* gnome_pilot_conduit_progress(conduit,?,?); */
 		}
 	}
 
@@ -512,7 +517,8 @@ gnome_pilot_conduit_standard_real_copy_from_pilot   (GnomePilotConduitStandard *
 		}
 		index++;
 		gnome_pilot_conduit_progress(GNOME_PILOT_CONDUIT(conduit),
-					     conduit->total_records,index);
+					     conduit->total_records,
+					     ++conduit->progress);
 	}
 
 	standard_abs_close_db_and_purge_local(conduit,dbinfo);
@@ -593,6 +599,7 @@ gnome_pilot_conduit_standard_real_synchronize       (GnomePilotConduitStandard *
 	if (conduit->num_local_records == -1) {
 		conduit->num_local_records = conduit->total_records;
 	}
+	conduit->total_progress += conduit->num_updated_local_records;
 	if (conduit->num_updated_local_records == -1) {
 		conduit->num_updated_local_records = conduit->total_records;
 	}
@@ -986,11 +993,13 @@ standard_abs_delete_from_pilot(GnomePilotConduitStandardAbs *conduit,
 	int err;
 	
 	g_message("gpilotd: deleting record %ld from pilot",remote->ID);
-
 	err = dlp_DeleteRecord(handle,db,0,remote->ID);
-	if (err<=0) {
+	if (err<0) {
 		g_warning("dlp_DeleteRecord returned %d",err);
 	}
+	/* skip this records during later sync_record call */
+	conduit->record_ids_to_ignore = g_slist_prepend(conduit->record_ids_to_ignore,
+							GINT_TO_POINTER(remote->ID));
 }
 
 /* updates or adds a local record to the pilot. Note, that after calling this,
@@ -1024,7 +1033,7 @@ standard_abs_add_to_pilot (GnomePilotConduitStandardAbs *conduit,
 			       remote->record,
 			       remote->length,
 			       &assigned_id);
-	if (err<=0) {
+	if (err<0) {
 		g_warning("dlp_WriteRecord returned %d",err);
 		return 0;
 	}
@@ -1054,7 +1063,8 @@ standard_abs_add_to_pilot (GnomePilotConduitStandardAbs *conduit,
 
   Call with SyncToRemote|SyncToLocal to synchronize both ways
 
-  Case 12, 13, 14, 15, 18, 19 and 20 are still missing, this basically
+  Case 13, 14, 15 unimplemented,
+  case 18, 19 and 20 are still missing, this basically
   renders archiving useless.
 
   eskil */
@@ -1112,10 +1122,9 @@ standard_abs_sync_record (GnomePilotConduitStandardAbs *conduit,
 	if (local) {
 		/* local record exists */
 		if (remote->archived) {
-#if 0	
 			switch(local->attr) {
 			case GnomePilotRecordModified:
-				if (remove->attr == GnomePilotRecordModified) {
+				if (remote->attr == GnomePilotRecordModified) {
 					if (gnome_pilot_conduit_standard_abs_compare(conduit,local,remote) != 0) {
 					 	/* CASE 14 */
 					} else {
@@ -1149,7 +1158,6 @@ standard_abs_sync_record (GnomePilotConduitStandardAbs *conduit,
 			case GnomePilotRecordModified:
 				break;
 			}
-#endif
 		} else {
 			switch (remote->attr) {
 			case GnomePilotRecordModified:
@@ -1215,10 +1223,14 @@ standard_abs_sync_record (GnomePilotConduitStandardAbs *conduit,
 					/* CASE 3 */
 					g_message("gpilotd: sync_record: case 3");
 					/* can be collapsed with case GnomePilotRecordDeleted */
-					if ( direction & SyncToRemote )
+					if ( direction & SyncToRemote ) {
 						standard_abs_delete_from_pilot(conduit,handle,db,remote);
-					if ( direction & SyncToLocal )
-						gnome_pilot_conduit_standard_abs_set_status (conduit, local, GnomePilotRecordDeleted);
+					}
+					if ( direction & SyncToLocal ) {
+						gnome_pilot_conduit_standard_abs_set_status (conduit, 
+											     local, 
+											     GnomePilotRecordDeleted);	
+					}
 					break;
 				default:
 					g_warning("gpilotd: sync_record: Unhandled sync case (a) Remote.attr = %d, local.attr = %d\n",
@@ -1324,7 +1336,8 @@ standard_abs_check_locally_deleted_records(GnomePilotConduitStandardAbs *conduit
 		g_message("gpilotd: locally deleted record...");
 		standard_abs_sync_record(conduit, handle, db, local, NULL, direction);
 		gnome_pilot_conduit_progress(GNOME_PILOT_CONDUIT(conduit),
-					     conduit->total_progress, ++conduit->progress);
+					     conduit->total_progress, 
+					     ++conduit->progress);
 	}
 	return 0;
 }
@@ -1353,7 +1366,8 @@ standard_abs_merge_to_remote (GnomePilotConduitStandardAbs *conduit,
 								       local, 
 								       assigned_id);
 			gnome_pilot_conduit_progress(GNOME_PILOT_CONDUIT(conduit),
-						     conduit->total_progress, ++conduit->progress);
+						     conduit->total_progress, 
+						     ++conduit->progress);
 		}
 
 	/* 
@@ -1364,7 +1378,8 @@ standard_abs_merge_to_remote (GnomePilotConduitStandardAbs *conduit,
 	while(gnome_pilot_conduit_standard_abs_iterate_specific (conduit, &local, GnomePilotRecordModified, 0)) {
 		standard_abs_sync_record(conduit, handle, db, local, NULL, direction);
 		gnome_pilot_conduit_progress(GNOME_PILOT_CONDUIT(conduit),
-					     conduit->total_progress,++conduit->progress);
+					     conduit->total_progress,
+					     ++conduit->progress);
 	}
 
 	return 0;
@@ -1378,11 +1393,13 @@ standard_abs_merge_to_local (GnomePilotConduitStandardAbs *conduit,
         int index;
 	int retval;
 	PilotRecord remote;
+	unsigned char buffer[0xffff];
 	
 	g_assert(conduit!=NULL);
 
 	index = 0;
 	retval = 0;
+	remote.record = buffer;
 
 	while (dlp_ReadRecordByIndex(handle,db,
 				     index,
@@ -1397,7 +1414,8 @@ standard_abs_merge_to_local (GnomePilotConduitStandardAbs *conduit,
 			standard_abs_sync_record (conduit, handle, db, NULL, &remote, direction);
 		index++;
 		gnome_pilot_conduit_progress(GNOME_PILOT_CONDUIT(conduit),
-					     conduit->total_progress,index);
+					     conduit->total_progress,
+					     index);
 	}
 	conduit->progress = index;
 
@@ -1433,7 +1451,8 @@ SlowSync (int handle,
 		standard_abs_sync_record (conduit, handle, db, NULL, &remote, SyncBothWays);
 		index++;
 		gnome_pilot_conduit_progress(GNOME_PILOT_CONDUIT(conduit),
-					     conduit->total_progress,index);
+					     conduit->total_progress,
+					     index);
 	}
 	conduit->progress = index;
 
@@ -1471,7 +1490,8 @@ FastSync (int handle,
 		standard_abs_compute_attr_field(&remote);
 		standard_abs_sync_record (conduit, handle, db, NULL, &remote, SyncBothWays);
 		gnome_pilot_conduit_progress(GNOME_PILOT_CONDUIT(conduit),
-					     conduit->total_progress,index);
+					     conduit->total_progress,
+					     index);
 	}
 	conduit->progress = index;
 
