@@ -23,6 +23,7 @@ GPilotContext *gpilot_context_new (void)
 	GPilotContext *retval;
 	
 	retval = (GPilotContext *)g_malloc(sizeof(GPilotContext));
+	retval->paused = FALSE;
 	retval->devices = NULL;
 	retval->pilots = NULL;
 	retval->user = NULL;
@@ -84,11 +85,7 @@ GPilotDevice *
 gpilot_device_new (void)
 {
   GPilotDevice *device;
-  device = (GPilotDevice *)g_malloc(sizeof(GPilotDevice));
-  device->name = NULL;
-  device->port = NULL;
-  device->speed = 0;
-  device->fd = 0;
+  device = g_new0(GPilotDevice,1);
   return device;
 }
 
@@ -109,17 +106,17 @@ gpilot_device_init (GPilotDevice *device, gint i)
 	g_free (device->port);
 	device->port = gnome_config_get_string("device");
 
-	device->fd=open(device->port,O_RDWR);
+	device->fd=open(device->port,O_RDWR|O_NOCTTY|O_NONBLOCK);
 	if(device->fd < 0) {
 		g_warning(_("Could not open device %s (%s): reason: \"%s\"."),
 			  device->name,device->port,
 			  g_strerror(errno));
-
+#ifdef GUI
 		gpilot_gui_warning_dialog(_("GnomePilot could not open device %s (%s).\n"
 					    "Reason: \"%s\"."),
 					  device->name,device->port,
 					  g_strerror(errno));
-
+#endif
 		g_free(device->name);
 		g_free(device->port);
 		close(device->fd);
@@ -136,8 +133,8 @@ gpilot_device_init (GPilotDevice *device, gint i)
 void
 gpilot_device_free (GPilotDevice *device)
 {
-  g_io_channel_close(device->io);
-  g_free(device->name);
+  if(device->io) g_io_channel_close(device->io);
+  g_free(device->name); 
   g_free(device->port);
   g_free(device);
 }
@@ -151,11 +148,12 @@ get_devices (void)
   final = n = gnome_config_get_int("/gnome-pilot.d/gpilotd/General/num_devices=0");
 
   if(n==0) {
-    g_warning("number of devices is configured to 0");
-    g_warning("Please run gpilotd-control-applet (use gnomecc) to configure.");    
+    g_warning(_("Number of devices is configured to 0"));
+#ifdef GUI
     gpilot_gui_warning_dialog(_("No cradles configure.\n"
 				"Please run gpilotd-control-applet (use gnomecc)\n" 
 				"to configure gnome-pilot."));    
+#endif
   }
 
   for(i=0;i<n;i++) {
@@ -184,12 +182,12 @@ get_pilots (void)
   n = gnome_config_get_int("/gnome-pilot.d/gpilotd/General/num_pilots=0");
 
   if(n==0) {
-    g_warning("number of pilots is configured to 0");
-    g_warning("Winging it anyways...");
-    g_warning("Please run gpilotd-control-applet (eg. use gnomecc) to configure.");
+    g_warning(_("Number of pilots is configured to 0"));
+#ifdef GUI
     gpilot_gui_warning_dialog(_("No pilots configured.\n"
 				"Please run gpilotd-control-applet (use gnomecc)\n" 
 				"to configure gnome-pilot."));    
+#endif
   }
 
 
