@@ -4,10 +4,11 @@
 #include "gpilot-userinfo.h"
 
 typedef struct GPilotContext {
-  GList *devices;
-  GList *pilots;
-  GPilotUser *user; /* Null in master global gpilotd */
-  guint32 sync_PC_Id; /* Different for every gpilotd instance */
+	gboolean paused;
+	GList *devices;
+	GList *pilots;
+	GPilotUser *user; /* Null in master global gpilotd */
+	guint32 sync_PC_Id; /* Different for every gpilotd instance */
 } GPilotContext;
 
 GPilotContext *gpilot_context_new (void);
@@ -21,7 +22,9 @@ typedef struct GPilotDevice {
 	/* These are used at runtime, to associate 
 	   an opened file with the device */
 	int fd;
-	GIOChannel *io;
+	GIOChannel *io;    /* the g_io_channel is used for the polling */
+	guint in_handle;   /* these two handles are used to be able to */
+	guint err_handle;  /* pause the daemon's polling by g_source_remove */
 } GPilotDevice;
 
 GPilotDevice *gpilot_device_new (void);
