@@ -10,19 +10,7 @@ GPilotPilot *
 gpilot_pilot_new (void)
 {
 	GPilotPilot *retval;
-
-	retval = (GPilotPilot *)g_malloc(sizeof(GPilotPilot));
-	if (!retval)
-		return NULL;
-	retval->name = NULL;
-	retval->passwd = NULL;
-	retval->number = 0;
-	retval->pilot_username = NULL;
-	retval->pilot_id = 0;
-	retval->trusted_users = NULL;
-	retval->sync_options.default_sync_action = 0;
-	retval->sync_options.basedir = NULL;
-  
+	retval = g_new0(GPilotPilot,1);
 	return retval;
 }
 
