@@ -16,28 +16,28 @@
 #include "gnome-pilot-conduit-standard.h"
 #include "gpilot-gui.h"
 
-#define PROGRESS_STEPPING 10
-#define USE_GMODULE 
+#include <libgpilotdCM/gnome-pilot-conduit-management.h>
+#include <libgpilotdCM/gnome-pilot-conduit-config.h>
 
-#ifndef USE_GMODULE
-/* we're using dlopen instead of g_module_open, include nessecary .h's */
-#include <dlfcn.h>
-#endif
+#define PROGRESS_STEPPING 5
+#define GPCM_LABEL "conduit_mgmt"
+#define GPCC_LABEL "conduit_cfg"
 
 /* File and Backup conduits are handled completely separately from the
    other conduits */
-/* Typedefs. */
+/*
 typedef struct GnomePilotConduitManagerInfo {
 	gchar *name;
 	GModule *dlhandle;
-	conduit_destroy_gpilot_conduit destroy_gpilot_conduit;
+	GnomePilotConduitDestroyFunc destroy_gpilot_conduit;
 
 	gchar *config_name;
 	GnomePilotConduitSyncType sync_type;
 	GnomePilotConduitSyncType first_sync_type;
 	gboolean slow;
 } GnomePilotConduitManagerInfo;
-
+*/
+/* Typedefs. */
 typedef struct {
 	GnomePilotConduit *conduit;
 	GPilotContext *context;
@@ -50,11 +50,7 @@ typedef struct {
 typedef gint (*iterate_func)(GnomePilotConduitStandard *,GnomePilotDBInfo *);
 
 /* Prototypes */
-
-static GnomePilotConduit *gpilot_load_conduit (guint32 pilotId,
-					       gchar *conduit_fn,
-					       GnomePilotConduitManagerInfo *cmaninfo);
-static void unload_foreach (gpointer *conduit, gpointer unused);
+static void unload_foreach (GnomePilotConduit *conduit, gpointer unused);
 static gint find_matching_conduit_compare_func (GnomePilotConduit *a,
 						GnomePilotDBInfo *b);
 static GnomePilotConduit *find_matching_conduit (GList *conlist, GnomePilotDBInfo *dbinfo);
@@ -74,6 +70,7 @@ static gint conduit_copy_from_pilot (GnomePilotConduitStandard *conduit,
 				     GnomePilotDBInfo *dbinfo);
 static gint conduit_sync_default (GnomePilotConduitStandard *conduit,
 			     GnomePilotDBInfo *dbinfo);
+static void gpilot_manager_save_databases(GnomePilotDBInfo *dbinfo, GSList *databases);
 /* This is defined in gpilotd.  But we need to use it... */
 gchar *pilot_name_from_id(guint32 id,GPilotContext *context);
 
@@ -142,31 +139,18 @@ static void
 conduit_progress_callback(GnomePilotConduit *conduit, 
 			  gint total,
 			  gint current,
-			  GnomePilotDBInfo *db) {
-	static gint top = 0;
-	gfloat tot_f,cur_f;
-	gint pct;
-
-	tot_f = total;
-	cur_f = current;
-	pct = abs(cur_f/(tot_f/100));
-	
-	if (pct < PROGRESS_STEPPING) top = 0; /* ensures resetting top */
-	if (pct >= top+PROGRESS_STEPPING) {
-		/* for every PROGRESS_STEPPING%, call the corba thingy */
-		top = pct;
-		orbed_notify_conduit_progress(db->pilotInfo->name,
-					      conduit,
-					      current,
-					      total);
-	}
+			  gchar *pilot_id) {
+	orbed_notify_conduit_progress(pilot_id,
+				      conduit,
+				      current,
+				      total);
 }
 
 static void 
 conduit_message_callback(GnomePilotConduit *conduit, 
 			 gchar *message,
-			 GnomePilotDBInfo *db) {
-	orbed_notify_conduit_message(db->pilotInfo->name,
+			 gchar *pilot_id) {
+	orbed_notify_conduit_message(pilot_id,
 				     conduit,
 				     message);
 }
@@ -174,41 +158,69 @@ conduit_message_callback(GnomePilotConduit *conduit,
 static void 
 conduit_error_callback(GnomePilotConduit *conduit, 
 		       gchar *message,
-		       GnomePilotDBInfo *db) {
-	orbed_notify_conduit_error(db->pilotInfo->name,
+			 gchar *pilot_id) {
+	orbed_notify_conduit_error(pilot_id,
 				   conduit,
 				   message);
 }
 
+/* This toggles the callbacks for signal. Call with set=TRUE to 
+   register the callbacks, and FALSE to remove them.
+   Remember to match these two calls!
+   NOTE: Not reentrant! lock when set, unlock when unset.
+   could potientially be reentrant since the pilot-name
+   would be unique even for multicradle sync
+*/
+static void 
+set_callbacks(gboolean set,GnomePilotConduit *conduit, gchar *pilot_name) 
+{
+	static guint progress_h = 0,
+		     error_h    = 0,
+		     message_h  = 0;
+	if (set==TRUE) {
+		if (progress_h != 0) g_warning("Internal inconsistency, see %s:%d",
+					       __FILE__,__LINE__);
+		
+		progress_h = gtk_signal_connect(GTK_OBJECT(conduit),"progress",
+						(GtkSignalFunc)conduit_progress_callback,
+						pilot_name);
+		
+		error_h = gtk_signal_connect(GTK_OBJECT(conduit),"error",
+					     (GtkSignalFunc)conduit_error_callback,
+					     pilot_name);
+		
+		message_h = gtk_signal_connect(GTK_OBJECT(conduit),"message",
+					       (GtkSignalFunc)conduit_message_callback,
+					       pilot_name);
+		gnome_pilot_conduit_set_stepping(conduit,PROGRESS_STEPPING);
+	} else {
+		if (progress_h == 0) g_warning("Internal inconsistency, see %s:%d",
+					       __FILE__,__LINE__);
+		
+		gtk_signal_disconnect(GTK_OBJECT(conduit),progress_h);
+		gtk_signal_disconnect(GTK_OBJECT(conduit),error_h);
+		gtk_signal_disconnect(GTK_OBJECT(conduit),message_h);
+		progress_h = 0;
+		error_h    = 0;
+		message_h  = 0;
+	}
+}
+
 /**************************************************************************/
 
 static void
 backup_foreach (GnomePilotConduitBackup *conduit, GnomePilotDBInfo *info)
 {
 
-	guint progress_h,error_h,message_h;
-
 	if (GNOME_IS_PILOT_CONDUIT_BACKUP (conduit) == FALSE) {
 		g_error(_("non-backup conduit in backup conduit list"));
 		return;
 	}
 	g_assert (info != NULL);
-
-	progress_h = gtk_signal_connect(GTK_OBJECT(conduit),"progress",
-					(GtkSignalFunc)conduit_progress_callback,
-					info);
-	error_h = gtk_signal_connect(GTK_OBJECT(conduit),"error",
-				     (GtkSignalFunc)conduit_error_callback,
-				     info);
-	message_h = gtk_signal_connect(GTK_OBJECT(conduit),"message",
-				       (GtkSignalFunc)conduit_message_callback,
-				       info);
-
+	
+	set_callbacks(TRUE,GNOME_PILOT_CONDUIT(conduit),info->pilotInfo->name);
 	gnome_pilot_conduit_backup_backup (conduit, info);
-
-	gtk_signal_disconnect(GTK_OBJECT(conduit),progress_h);
-	gtk_signal_disconnect(GTK_OBJECT(conduit),error_h);
-	gtk_signal_disconnect(GTK_OBJECT(conduit),message_h);
+	set_callbacks(FALSE,GNOME_PILOT_CONDUIT(conduit),info->pilotInfo->name);
 }
 
 static gint
@@ -221,7 +233,8 @@ iterate_dbs (gint pfd,
 	     GPilotContext *context)
 {
 	GnomePilotDBInfo dbinfo;
-	GnomePilotConduit *conduit;
+	GnomePilotConduit *conduit = NULL;
+	GSList *db_list = NULL;
 	int index = 0;
 	int error = 0;
 
@@ -233,12 +246,17 @@ iterate_dbs (gint pfd,
 	while(1) {
 		if(dlp_OpenConduit(pfd) < 0) {
 			g_error("Unable to open conduit!");
-			return -1;
+			error = -1;
+			break;
 		}
 		/* load next dbinfo block */
-		if(dlp_ReadDBList(pfd, 0, 0x80, index, PI_DBINFO (&dbinfo)) < 0)
+		if(dlp_ReadDBList(pfd, 0, 0x80, index, PI_DBINFO (&dbinfo)) < 0) {
+			/* is <0, there are no more databases, break
+                           out so we can save the list */
 			break;
+		}
 		index = PI_DBINFO (&dbinfo)->index + 1;
+		db_list = g_slist_append(db_list,g_strdup(PI_DBINFO (&dbinfo)->name));
 		/*
 		g_message("DB: %20s: Exclude=%3s, Backup=%3s Flags=0x%8.8x Creator=0x%lx",
 			   PI_DBINFO(&dbinfo)->name,
@@ -249,7 +267,7 @@ iterate_dbs (gint pfd,
 		*/
 		/* check if the base is marked as being excluded from sync */
 		if (!(PI_DBINFO(&dbinfo)->miscFlags & dlpDBMiscFlagExcludeFromSync)) {
-			guint progress_h,error_h,message_h;
+		        gchar *pilot_name;
 
 			dbinfo.pu = pu;
 			dbinfo.pilot_socket = pfd;
@@ -270,25 +288,18 @@ iterate_dbs (gint pfd,
 			   the signals. The signals aren't set before, since we 
 			   want to pass dbinfo as the userdata, and we don't do it
 			   in "iter", since it's a parameterized funtion */
-			progress_h = gtk_signal_connect(GTK_OBJECT(conduit),"progress",
-							(GtkSignalFunc)conduit_progress_callback,
-							&dbinfo);
-			error_h = gtk_signal_connect(GTK_OBJECT(conduit),"error",
-						     (GtkSignalFunc)conduit_error_callback,
-						     &dbinfo);
-			message_h = gtk_signal_connect(GTK_OBJECT(conduit),"message",
-						       (GtkSignalFunc)conduit_message_callback,
-						       &dbinfo);
+      	                pilot_name = pilot_name_from_id(dbinfo.pu->userID,context);
+			set_callbacks(TRUE,GNOME_PILOT_CONDUIT(conduit),pilot_name);
 			error = iter (GNOME_PILOT_CONDUIT_STANDARD (conduit), &dbinfo);
-
-			gtk_signal_disconnect(GTK_OBJECT(conduit),progress_h);
-			gtk_signal_disconnect(GTK_OBJECT(conduit),error_h);
-			gtk_signal_disconnect(GTK_OBJECT(conduit),message_h);
+			set_callbacks(FALSE,GNOME_PILOT_CONDUIT(conduit),pilot_name);
+			g_free(pilot_name);
 		} else {
 			LOG("Base %s is to be ignored by sync",PI_DBINFO(&dbinfo)->name);
 			continue;
 		}
 	}
+
+	gpilot_manager_save_databases(&dbinfo,db_list);
 	return error;
 }
 
@@ -299,7 +310,7 @@ conduit_synchronize(GnomePilotConduitStandard *conduit, GnomePilotDBInfo *dbinfo
 	g_return_val_if_fail (conduit != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PILOT_CONDUIT_STANDARD (conduit), -1);
 	if(gnome_pilot_conduit_get_name(GNOME_PILOT_CONDUIT(conduit))!=NULL)
-		gpilot_add_log_entry(dbinfo->pilot_socket,"Synchronizing %s\n",
+		gpilot_add_log_entry(dbinfo->pilot_socket,_("Synchronizing %s\n"),
 				     gnome_pilot_conduit_get_name(GNOME_PILOT_CONDUIT(conduit)));
 	orbed_notify_conduit_start(dbinfo->pilotInfo->name,
 				   GNOME_PILOT_CONDUIT(conduit),
@@ -385,7 +396,8 @@ conduit_merge_from_pilot(GnomePilotConduitStandard *conduit, GnomePilotDBInfo *d
 static gint
 conduit_sync_default(GnomePilotConduitStandard *conduit, GnomePilotDBInfo *dbinfo)
 {
-	GnomePilotConduitManagerInfo *manager;
+	GnomePilotConduitConfig *conduit_config;
+	GnomePilotConduitManagement *manager;
 	GnomePilotConduitSyncType action;
 
 	g_return_val_if_fail (conduit != NULL, -1);
@@ -393,18 +405,19 @@ conduit_sync_default(GnomePilotConduitStandard *conduit, GnomePilotDBInfo *dbinf
 	g_return_val_if_fail (dbinfo!=NULL, -1);
 	g_return_val_if_fail (dbinfo->pu!=NULL, -1);
 
-	manager = gtk_object_get_data (GTK_OBJECT (conduit), "manager_info");
-	g_return_val_if_fail(manager!=NULL,-1);
-	if (manager->first_sync_type != GnomePilotConduitSyncTypeNotSet) {
-		g_message(_("Conduit %s's first synchronization..."),manager->name);
-		gpilot_remove_first_sync_settings(dbinfo->pu->userID,
-						  manager->config_name);
-		action = manager->first_sync_type;
-		if(manager->slow==TRUE) {
+	conduit_config = gtk_object_get_data (GTK_OBJECT (conduit), GPCC_LABEL);
+	manager = gtk_object_get_data (GTK_OBJECT (conduit), GPCM_LABEL);
+	g_return_val_if_fail(conduit_config!=NULL,-1);
+	if (conduit_config->first_sync_type != GnomePilotConduitSyncTypeNotSet) {
+		g_message(_("Conduit %s's first synchronization..."),
+			  gnome_pilot_conduit_management_get_name(manager));
+		gnome_pilot_conduit_config_remove_first_sync(conduit_config);
+		action = conduit_config->first_sync_type;
+		if(conduit_config->first_slow==TRUE) {
 			gnome_pilot_conduit_standard_set_slow(conduit);
 		}
 	} else {
-		action = manager->sync_type;
+		action = conduit_config->sync_type;
 	}
 		
 	switch (action) {
@@ -419,14 +432,18 @@ conduit_sync_default(GnomePilotConduitStandard *conduit, GnomePilotDBInfo *dbinf
 	case GnomePilotConduitSyncTypeMergeFromPilot:
 		return conduit_merge_from_pilot (conduit, dbinfo);
 	default:
-		g_error (_("unknown syncing action (%d = %s).\n"),action,
-			 gnome_pilot_conduit_sync_type_int_to_str(action));
+		g_warning (_("unknown syncing action (%d = %s).\n"),action,
+			   gnome_pilot_conduit_sync_type_int_to_str(action));
 		return -1;
 	}
 }
 
 static void
 install_db_foreach (GPilotRequest *req, GnomePilotInstallCarrier *carrier) {
+	gchar *pilot_name;
+
+	pilot_name = pilot_name_from_id(carrier->pu->userID,carrier->context);
+	set_callbacks(TRUE,carrier->conduit,pilot_name);
 	switch(req->type) {
 	case GREQ_INSTALL:
 		gnome_pilot_conduit_file_install_db (GNOME_PILOT_CONDUIT_FILE (carrier->conduit),
@@ -442,6 +459,8 @@ install_db_foreach (GPilotRequest *req, GnomePilotInstallCarrier *carrier) {
 		g_warning (_("%s this request.type == %d is not a install/restore"), __PRETTY_FUNCTION__, req->type);
 		break;
 	}
+	set_callbacks(FALSE,carrier->conduit,pilot_name);
+	g_free(pilot_name);
 	orbed_notify_completion (&req);
 };
 
@@ -452,6 +471,7 @@ install_foreach (GnomePilotConduit *conduit,
 	GList *request_list;
 
 	g_return_if_fail (conduit != NULL);
+	g_return_if_fail (info != NULL);
 	g_return_if_fail (GNOME_IS_PILOT_CONDUIT_FILE (conduit));
 
 	info->conduit = conduit;
@@ -480,108 +500,44 @@ static void
 do_restore_foreach(GPilotRequest *req, GnomePilotInstallCarrier *carrier)
 {
 	GList *iterator;
+	gchar *pilot_name;
+	
 	g_assert(carrier!=NULL);
 
+	pilot_name = pilot_name_from_id(carrier->pu->userID,carrier->context);
 	/* FIXME: if carrier.conduit_list is NULL, the user hasn't
 	  enabled any file conduits. The corba restore call should
 	  throw an exception or something.  */
 	iterator = carrier->conduit_list;
 	while( iterator != NULL ) {
-		/* FIXME: Do some amazing stuff here ! */
 		GnomePilotConduitFile *conduit;
 		conduit = GNOME_PILOT_CONDUIT_FILE(iterator->data);
+		set_callbacks(TRUE,GNOME_PILOT_CONDUIT(conduit),pilot_name);
 		gnome_pilot_conduit_file_restore_directory(conduit,carrier->pilot_socket,
 							   req->parameters.restore.directory);
 		iterator = iterator->next;
+		set_callbacks(FALSE,GNOME_PILOT_CONDUIT(conduit),pilot_name);
 	}
 
-
+	g_free(pilot_name);
 	orbed_notify_completion (&req);
 }
 
 static void
-unload_foreach(gpointer *conduit,
+unload_foreach(GnomePilotConduit *conduit,
 	       gpointer unused)
 {
-	GnomePilotConduitManagerInfo *manager;
+	GnomePilotConduitManagement *manager;
 	g_return_if_fail (conduit != NULL);
 	g_return_if_fail (GNOME_IS_PILOT_CONDUIT (conduit));
 	
-	manager = gtk_object_get_data (GTK_OBJECT (conduit), "manager_info");
+	manager = gtk_object_get_data (GTK_OBJECT (conduit), GPCM_LABEL);
 	g_assert(manager!=NULL);
-	manager->destroy_gpilot_conduit (GNOME_PILOT_CONDUIT (conduit));
-#ifdef USE_GMODULE
-	g_module_close (manager->dlhandle);
-#else
-	dlclose (manager->dlhandle);
-#endif
-	g_free(manager->name);
-	g_free(manager->config_name);
-	g_free(manager);
-}
-
-static GnomePilotConduit *
-gpilot_load_conduit(guint32 pilotId,
-		    gchar *conduit_fn,
-		    GnomePilotConduitManagerInfo *cmaninfo)
-{
-	GModule *dlhandle;
-	GnomePilotConduit *conduit;
-	conduit_get_gpilot_conduit get_gpilot_conduit;
-	conduit_destroy_gpilot_conduit destroy_gpilot_conduit;
-
-	g_assert(cmaninfo!=NULL);
-	g_assert(conduit_fn!=NULL);
-
-#ifdef USE_GMODULE
-	if((dlhandle = g_module_open(conduit_fn, G_MODULE_BIND_LAZY))) {
-		if(!g_module_symbol(dlhandle,"conduit_get_gpilot_conduit",
-				    (gpointer) &get_gpilot_conduit)) {
-			g_error(_("Malformed conduit %s, cannot find "
-				  "conduit_get_gpilot_conduit: %s"),conduit_fn,g_module_error());
-			g_module_close(dlhandle);
-			return NULL;
-		}
-
-		if(!g_module_symbol(dlhandle,"conduit_destroy_gpilot_conduit",
-				    (gpointer) &destroy_gpilot_conduit)) {
-			g_error(_("Malformed conduit %s, cannot find "
-				  "conduit_destroy_gpilot_conduit: %s"),conduit_fn,g_module_error());
-			g_module_close(dlhandle);
-			return NULL;
-		}
-#else
-	if((dlhandle = dlopen(conduit_fn, RTLD_LAZY))) {
-		if((get_gpilot_conduit = dlsym(dlhandle,"conduit_get_gpilot_conduit"))==NULL) {
-			g_error(_("Malformed conduit %s, cannot find "
-				  "conduit_get_gpilot_conduit: %s"),conduit_fn,g_module_error());
-			dlclose(dlhandle);
-			return NULL;
-		}
-
-		if((destroy_gpilot_conduit = dlsym(dlhandle,"conduit_destroy_gpilot_conduit")) == NULL) {
-
-			g_error(_("Malformed conduit %s, cannot find "
-				  "conduit_destroy_gpilot_conduit: %s"),conduit_fn,g_module_error());
-			dlclose(dlhandle);
-			return NULL;
-		}
-#endif
-
-		conduit = get_gpilot_conduit(pilotId);
-		cmaninfo->dlhandle = dlhandle;
-		cmaninfo->name = g_strdup (gnome_pilot_conduit_get_name (conduit));
-		cmaninfo->destroy_gpilot_conduit = destroy_gpilot_conduit;
-		gtk_object_set_data (GTK_OBJECT (conduit), "manager_info", cmaninfo);
-	} else {
-#ifdef USE_GMODULE
-		g_warning(_("Unable to open conduit %s: %s"),conduit_fn,g_module_error());
-#else
-		g_warning(_("Unable to open conduit %s: %s"),conduit_fn,dlerror());
-#endif
-		return NULL;
-	}
-	return conduit;
+	/* FIXME: _destroy_conduit doesn't g_module_close... is this an issue ? 
+	 note: the shlib loader in gnome_pilot_conduit_management now closes
+	after loading the methods. */
+	gnome_pilot_conduit_management_destroy_conduit(manager,&conduit);
+	gnome_pilot_conduit_management_destroy(manager);
 }
 
 
@@ -592,12 +548,12 @@ gpilot_load_conduits (GPilotPilot *pilot,
 		      GList **flist)
 {
 	int i, cnt;
-	gchar *conduit_fn, *conduit_sync_type_str;
 	gchar *prefix;
 	GnomePilotConduit *conduit;
 	guint32 pilotId;
 	gchar **conduit_name;
-	GnomePilotConduitManagerInfo *cmaninfo;
+	GnomePilotConduitManagement *manager;
+	GnomePilotConduitConfig *conduit_config;
 
 	pilotId = pilot->pilot_id;
 
@@ -611,58 +567,41 @@ gpilot_load_conduits (GPilotPilot *pilot,
 	gnome_config_get_vector("conduits",&cnt,&conduit_name);
 	gnome_config_pop_prefix();
 	g_free(prefix);
-	g_message(_("Loading %d conduits..."),cnt);
+	g_message(_("Instantiating %d conduits..."),cnt);
 	for(i = 0;i<cnt;i++) {
-		prefix = g_strdup_printf("/gnome-pilot.d/conduits%d/%s/",
-			   pilot->pilot_id,conduit_name[i]);
-		if(gnome_config_has_section(prefix)) {
-			cmaninfo = g_new0(GnomePilotConduitManagerInfo,1);
-			cmaninfo->config_name = g_strdup(conduit_name[i]);
-			gnome_config_push_prefix(prefix);
-			conduit_fn = gnome_config_get_string("libfile");
+		gint err;
+		manager = gnome_pilot_conduit_management_new(conduit_name[i],GNOME_PILOT_CONDUIT_MGMT_ID);
+		if (manager == NULL) {
+			g_message(_("Uknown conduit \"%s\" in configure!"),conduit_name[i]);			
+			gpilot_gui_warning_dialog(_("Uknown conduit \"%s\" in configure!"),conduit_name[i]); 
+			continue;
+		}
+		conduit_config = gnome_pilot_conduit_config_new(manager,pilot->pilot_id);
+		gnome_pilot_conduit_config_load_config(conduit_config);
+		err = gnome_pilot_conduit_management_instantiate_conduit(manager,pilotId,&conduit);
+		if (err != GNOME_PILOT_CONDUIT_MGMT_OK || conduit == NULL) {
+			g_message(_("Loading conduit \"%s\" failed!"),conduit_name[i]);
+			gpilot_gui_warning_dialog(_("Loading conduit \"%s\" failed!\n"),conduit_name[i]);
+			gnome_pilot_conduit_config_destroy(conduit_config);
+			gnome_pilot_conduit_management_destroy(manager);
+			continue;
+		}
 
-			conduit_sync_type_str = gnome_config_get_string("sync_type");
-			cmaninfo->sync_type = gnome_pilot_conduit_sync_type_str_to_int (conduit_sync_type_str);
-			g_free(conduit_sync_type_str);
+		gtk_object_set_data (GTK_OBJECT (conduit), GPCM_LABEL, manager);
+		gtk_object_set_data (GTK_OBJECT (conduit), GPCC_LABEL, conduit_config);
 
-			conduit_sync_type_str = gnome_config_get_string("first_sync_type");
-			if (conduit_sync_type_str != NULL) {
-				cmaninfo->first_sync_type = gnome_pilot_conduit_sync_type_str_to_int (conduit_sync_type_str);
-				g_free(conduit_sync_type_str);
-			} else
-				cmaninfo->first_sync_type = GnomePilotConduitSyncTypeNotSet;
-			cmaninfo->slow = gnome_config_get_bool("slow_sync");
-
-
-			conduit = gpilot_load_conduit(pilotId,conduit_fn,cmaninfo);
-			if (conduit == NULL) {
-				g_message(_("Loading conduit \"%s\" failed!"),conduit_name[i]);
-				gpilot_gui_warning_dialog(_("Loading conduit \"%s\" failed!\n"
-							    "It is likely a problem with the file\n"
-							    "%s"),conduit_name[i],conduit_fn);
-				g_free(conduit_fn);
-				continue;
-			}
-
-			if (GNOME_IS_PILOT_CONDUIT_BACKUP (conduit)) {
-				*blist = g_list_append (*blist, conduit);
-			} else if (GNOME_IS_PILOT_CONDUIT_FILE (conduit)) {
-				*flist = g_list_append (*flist, conduit);
-			} else if (GNOME_IS_PILOT_CONDUIT_STANDARD (conduit)) {
-				*clist = g_list_append (*clist, conduit);
-			} else {
-				g_warning("Error in conduits definition file for pilot %d, %s is not a valid conduit",
-					  pilot->number,
-					  conduit_name[i]);
-			}
-			g_free(conduit_fn);
-			gnome_config_pop_prefix();
+		if (GNOME_IS_PILOT_CONDUIT_BACKUP (conduit)) {
+			*blist = g_list_append (*blist, conduit);
+		} else if (GNOME_IS_PILOT_CONDUIT_FILE (conduit)) {
+			*flist = g_list_append (*flist, conduit);
+		} else if (GNOME_IS_PILOT_CONDUIT_STANDARD (conduit)) {
+			*clist = g_list_append (*clist, conduit);
 		} else {
-			g_warning("Error in conduits definition file for pilot %d, %s is not a valid section",
+			g_warning("Error in conduits definition file for pilot %d, %s is not a valid conduit",
 				  pilot->number,
 				  conduit_name[i]);
 		}
-		g_free(prefix);
+
 		g_free(conduit_name[i]);
 	}
 	g_free(conduit_name);
@@ -923,56 +862,47 @@ void
 gpilot_add_log_entry(int pilot_socket,
 		     gchar *entry, ...) 
 {
-	gchar *e;
+	gchar *e,*f;
 
-	/* Pilot link before 0.9.3 has a bug in dlp_AddSyncLogEntry,
-	 that eats last char (the \n) from input. Thus we add a space
-	 if pilot link is an older version */
-
-#if PILOT_LINK_VERSION == 0
-  #if PILOT_LINK_MAJOR == 9
-    #if PILOT_LINK_MINOR < 3
-      /* pilot-link version less then 0.9.3 */
-      #define PILOT_NAME_NEED_SPACE
-    #endif /* PILOT_LINK_MINOR < 3 */
-  #else /* PILOT_LINK_MAJOR == 9 */
-    #if PILOT_LINK_MAJOR < 9
-      /* "pilot-link version less then 0.9.x */
-      #define PILOT_NAME_NEED_SPACE
-    #endif /* PILOT_LINK_MAJOR < 9 */
-  #endif /* PILOT_LINK_MAJOR == 9 */
-#endif /* PILOT_LINK_VERSION == 0 */
-
-#ifdef PILOT_NAME_NEED_SPACE
-	gchar *f;
-#endif
 	va_list ap;
 
 	va_start(ap,entry);
 	e = g_strdup_vprintf(entry,ap);
-#ifdef PILOT_NAME_NEED_SPACE
 	f = g_strdup_printf("%s ",e);
 	dlp_AddSyncLogEntry(pilot_socket,f);
-	g_free (f);
-#else
-	dlp_AddSyncLogEntry(pilot_socket,e);
-#endif
 	g_free (e);
+	g_free (f);
 }
 
-void
-gpilot_remove_first_sync_settings(guint32 pilot_id,
-				  gchar *config_name) 
-{
-	gchar *prefix;
-	prefix = g_strdup_printf("/gnome-pilot.d/conduits%d/%s/",pilot_id,config_name);
-	if (gnome_config_has_section(prefix)) {
-		gnome_config_push_prefix(prefix);
-		gnome_config_clean_key("first_sync_type");
-		gnome_config_clean_key("slow_sync");
-		gnome_config_pop_prefix();
-		gnome_config_sync();
+/**
+   save the list of databases this pilot has
+ */
+void 
+gpilot_manager_save_databases(GnomePilotDBInfo *dbinfo, 
+			      GSList *databases) {
+	gchar *pfx;
+	char **charlist;
+	int cnt=0;
+	GSList *ptr;
+
+	charlist = g_new0(char*,g_slist_length(databases)+1);
+
+	for (ptr = databases; ptr != NULL; ptr = ptr->next) {
+		charlist[cnt] = ptr->data;
+		cnt++;
 	}
-	g_free(prefix);
 
+	pfx = g_strdup_printf("/gnome-pilot.d/PilotCache%d/Databases/",
+			      dbinfo->pilotInfo->pilot_id);
+	gnome_config_push_prefix(pfx);
+	gnome_config_set_vector("databases",
+				g_slist_length(databases),
+				(const char**)charlist);
+	gnome_config_pop_prefix();
+	gnome_config_sync();
+	/* FIXME: is this okay ? */
+	g_slist_foreach(databases,(GFunc)g_free,NULL);
+	g_free(pfx);
+	g_free(charlist);
+	g_slist_free(databases);
 }
