@@ -10,6 +10,8 @@
 #include <unistd.h>
 #include "gnome-pilot-conduit.h"
 #include "gnome-pilot-conduit-standard.h"
+#include <sys/types.h>
+#include <signal.h>
 
 /********************************************************************************/
 /* Defines                                                                      */
@@ -201,12 +203,41 @@ get_client_id(const GNOME_Pilot_Client cb)
 	return retval;
 }
 
+
+static void
+pause_device(GPilotDevice *device,gpointer data)
+{
+	g_source_remove(device->in_handle);
+	g_source_remove(device->err_handle);
+}
+
 /********************************************************************************/
 
+static void
+gpilotd_corba_pause(GNOME_Pilot_Daemon obj,
+		    CORBA_boolean on_off,
+		    CORBA_Environment *ev)  
+{
+	g_message("corba: gpilotd_corba_pause(%s)",on_off==TRUE?"true":"false");
+	if (orb_context->gpilotd_context->paused == on_off) return;
+
+	orb_context->gpilotd_context->paused = on_off;
+
+	if (orb_context->gpilotd_context->paused) {
+		g_list_foreach(orb_context->gpilotd_context->devices,
+			      (GFunc)pause_device,
+			       NULL);
+	} else {
+		kill(getpid(),SIGHUP);
+	}
+	
+	return;
+}
+
 static CORBA_unsigned_long
 gpilotd_corba_request_install(GNOME_Pilot_Daemon obj,
 			      const GNOME_Pilot_Client cb,
-			      const CORBA_char *pilot_id,
+			      const CORBA_char *pilot_name,
 			      const CORBA_char *filename,
 			      const CORBA_char *description,
 			      const GNOME_Pilot_Survival survival,
@@ -218,15 +249,15 @@ gpilotd_corba_request_install(GNOME_Pilot_Daemon obj,
 	
 	GET_AND_CHECK_CLIENT_ID(client_id,cb,0);
 
-	LOG("corba: request_install(pilot_id=%s (%d),filename=%s,survival=%d,timeout=%d)",
-	    pilot_id,
-	    pilot_id_from_name(pilot_id,orb_context->gpilotd_context),
+	LOG("corba: request_install(pilot_name=%s (%d),filename=%s,survival=%d,timeout=%d)",
+	    pilot_name,
+	    pilot_id_from_name(pilot_name,orb_context->gpilotd_context),
 	    filename,
 	    survival,
 	    timeout);
 
 	req.type = GREQ_INSTALL;
-	req.pilot_id = pilot_id_from_name(pilot_id,orb_context->gpilotd_context);
+	req.pilot_id = pilot_id_from_name(pilot_name,orb_context->gpilotd_context);
 
 	/* FIXME: oops, this exception requires some parameters, not setting them causes
 	   ORBit to crash&burn:
@@ -234,7 +265,7 @@ gpilotd_corba_request_install(GNOME_Pilot_Daemon obj,
 	*/
 	if(req.pilot_id==0) {
 		g_free(client_id);
-		gpthrow_with_settings(UnknownPilot,0,exn,exn->pilotId = CORBA_string_dup(pilot_id));
+		gpthrow_with_settings(UnknownPilot,0,exn,exn->pilotId = CORBA_string_dup(pilot_name));
 	}
 	if(access(filename,R_OK)) {
 		g_free(client_id);
@@ -256,39 +287,47 @@ gpilotd_corba_request_install(GNOME_Pilot_Daemon obj,
 static CORBA_unsigned_long
 gpilotd_corba_request_restore(GNOME_Pilot_Daemon obj,
 			      const GNOME_Pilot_Client cb,
-			      const CORBA_char *pilot_id,
-			      const CORBA_char *directory,
+			      const CORBA_char *pilot_name,
+			      const CORBA_char *_directory,
 			      const GNOME_Pilot_Survival survival,
 			      const CORBA_unsigned_long timeout,
 			      CORBA_Environment *ev)
 {
 	GPilotRequest req;
+	gchar *directory;
 	struct stat statbuf;
 	char *client_id;
 	
 	GET_AND_CHECK_CLIENT_ID(client_id,cb,0);
 
-	LOG("corba: request_restore(pilot_id=%s (%d), directory=%s,survival=%d,timeout=%d)",
-	    pilot_id,
-	    pilot_id_from_name(pilot_id,orb_context->gpilotd_context),
+	directory = g_strdup(_directory);
+	/* FIXME: should be a generic / remover */
+	if (directory[strlen(directory)-1]=='/')
+		directory[strlen(directory)-1]=0;
+	LOG("corba: request_restore(pilot_name=%s (%d), directory=%s,survival=%d,timeout=%d)",
+	    pilot_name,
+	    pilot_id_from_name(pilot_name,orb_context->gpilotd_context),
 	    directory,
 	    survival,
 	    timeout);
 
 	req.type = GREQ_RESTORE;
-	req.pilot_id = pilot_id_from_name(pilot_id,orb_context->gpilotd_context);
+	req.pilot_id = pilot_id_from_name(pilot_name,orb_context->gpilotd_context);
 
 	if(req.pilot_id==0) {
 		g_free(client_id);
-		gpthrow_with_settings(UnknownPilot,0,exn,exn->pilotId = CORBA_string_dup(pilot_id));
+		g_free(directory);
+		gpthrow_with_settings(UnknownPilot,0,exn,exn->pilotId = CORBA_string_dup(pilot_name));
 	}
-	if(stat(directory,&statbuf)!=0) {
+	if(stat(directory,&statbuf)==-1) {
 		g_free(client_id);
-		g_message("here");
+		g_free(directory);
+
 		gpthrow(MissingFile,0);
 	}
 	if(S_ISDIR(statbuf.st_mode) && access(directory,R_OK)!=0) {
 		g_free(client_id);
+		g_free(directory);
 		g_message("or here access=%d, S_ISDIR=%d",access(directory,R_OK),S_ISDIR(statbuf.st_mode));
 		gpthrow(MissingFile,0);
 	}
@@ -298,6 +337,62 @@ gpilotd_corba_request_restore(GNOME_Pilot_Daemon obj,
 
 	req.parameters.restore.directory = g_strdup(directory);
 
+	g_free(directory);
+	g_free(client_id);
+
+	return gpc_queue_store_request(req);
+}
+
+static CORBA_unsigned_long
+gpilotd_corba_request_revive(GNOME_Pilot_Daemon obj,
+			     const GNOME_Pilot_Client cb,
+			     const CORBA_char *cradle,
+			     const GNOME_Pilot_PilotUser *user,
+			     const CORBA_char *_directory,
+			     const GNOME_Pilot_Survival survival,
+			     const CORBA_unsigned_long timeout,
+			     CORBA_Environment *ev)
+{
+	GPilotRequest req;
+	gchar *directory;
+	struct stat statbuf;
+	char *client_id;
+	
+	GET_AND_CHECK_CLIENT_ID(client_id,cb,0);
+
+	directory = g_strdup(_directory);
+	/* FIXME: should be a generic / remover */
+	if (directory[strlen(directory)-1]=='/')
+		directory[strlen(directory)-1]=0;
+	LOG("corba: request_revive(cradle=%s, directory=%s,survival=%d,timeout=%d)",
+	    cradle,
+	    directory,
+	    survival,
+	    timeout);
+
+	req.type = GREQ_REVIVE;
+
+	if(stat(directory,&statbuf)==-1) {
+		g_free(client_id);
+		g_free(directory);
+		gpthrow(MissingFile,0);
+	}
+	if(S_ISDIR(statbuf.st_mode) && access(directory,R_OK)!=0) {
+		g_free(client_id);
+		g_free(directory);
+		g_message("or here access=%d, S_ISDIR=%d",access(directory,R_OK),S_ISDIR(statbuf.st_mode));
+		gpthrow(MissingFile,0);
+	}
+	req.timeout = survival==GNOME_Pilot_PERSISTENT?0:timeout;
+	req.cradle = NULL;
+	req.client_id = g_strdup(client_id);
+
+	req.parameters.revive.directory = g_strdup(directory);
+	req.parameters.revive.password = NULL;
+	req.parameters.revive.user_id = g_strdup(user->username);
+	req.parameters.revive.pilot_id = user->userID;
+
+	g_free(directory);
 	g_free(client_id);
 
 	return gpc_queue_store_request(req);
@@ -306,7 +401,7 @@ gpilotd_corba_request_restore(GNOME_Pilot_Daemon obj,
 static CORBA_unsigned_long
 gpilotd_corba_request_conduit(GNOME_Pilot_Daemon _obj, 
 			      const GNOME_Pilot_Client cb,
-			      const CORBA_char * pilot_id, 
+			      const CORBA_char * pilot_name, 
 			      const CORBA_char * conduit_name, 
 			      const GNOME_Pilot_ConduitOperation operation,
 			      const GNOME_Pilot_Survival survival, 
@@ -319,14 +414,14 @@ gpilotd_corba_request_conduit(GNOME_Pilot_Daemon _obj,
 	GET_AND_CHECK_CLIENT_ID(client_id,cb,0);
   
 	LOG("corba: request_conduit(pilot=%s (%d), conduit=%s)",
-	    pilot_id,
-	    pilot_id_from_name(pilot_id,orb_context->gpilotd_context),
+	    pilot_name,
+	    pilot_id_from_name(pilot_name,orb_context->gpilotd_context),
 	    conduit_name);
-	req.pilot_id = pilot_id_from_name(pilot_id,orb_context->gpilotd_context);
+	req.pilot_id = pilot_id_from_name(pilot_name,orb_context->gpilotd_context);
 
 	if(req.pilot_id==0) {
 		g_free(client_id);
-		gpthrow_with_settings(UnknownPilot,0,exn,exn->pilotId = CORBA_string_dup(pilot_id));
+		gpthrow_with_settings(UnknownPilot,0,exn,exn->pilotId = CORBA_string_dup(pilot_name));
 	}
 
 	/* FIXME: check for conduit existence, throw exn if not */
@@ -396,7 +491,7 @@ gpilotd_corba_get_user_info(GNOME_Pilot_Daemon obj,
 static CORBA_unsigned_long
 gpilotd_corba_get_system_info(GNOME_Pilot_Daemon obj,
 			      const GNOME_Pilot_Client cb,
-			      const CORBA_char *pilot_id, 
+			      const CORBA_char *pilot_name, 
 			      const GNOME_Pilot_Survival survival,
 			      const CORBA_unsigned_long timeout,
 			      CORBA_Environment *ev) 
@@ -406,13 +501,13 @@ gpilotd_corba_get_system_info(GNOME_Pilot_Daemon obj,
 	GET_AND_CHECK_CLIENT_ID(client_id,cb,0);
 	
 	LOG("corba: get_system_info(pilot=%s,survival=%d,timeout=%d)",
-	    pilot_id,survival,timeout);
+	    pilot_name,survival,timeout);
 	
 	req.type = GREQ_GET_SYSINFO;
-	req.pilot_id = pilot_id_from_name(pilot_id,orb_context->gpilotd_context);
+	req.pilot_id = pilot_id_from_name(pilot_name,orb_context->gpilotd_context);
 	if(req.pilot_id==0) {
 		g_free(client_id);
-		gpthrow_with_settings(UnknownPilot,0,exn,exn->pilotId = CORBA_string_dup(pilot_id));
+		gpthrow_with_settings(UnknownPilot,0,exn,exn->pilotId = CORBA_string_dup(pilot_name));
 	}
 	req.timeout = survival==GNOME_Pilot_PERSISTENT?0:timeout;
 	req.cradle = NULL;
@@ -458,6 +553,7 @@ gpilotd_corba_set_user_info(GNOME_Pilot_Daemon obj,
 	return gpc_queue_store_request(req);
 }
 
+#if 0
 static CORBA_unsigned_long
 gpilotd_corba_set_user_info_if_not_set(GNOME_Pilot_Daemon obj,
 				       GNOME_Pilot_PilotUser *user,
@@ -495,11 +591,12 @@ gpilotd_corba_set_user_info_if_not_set(GNOME_Pilot_Daemon obj,
 
 	return gpc_queue_store_request(req);
 }
+#endif
 
 static void
 gpilotd_corba_monitor_on(GNOME_Pilot_Daemon obj,
 			 const GNOME_Pilot_Client cb, 
-			 const CORBA_char *pilot_id, 
+			 const CORBA_char *pilot_name, 
 			 CORBA_Environment *ev) 
 {
 	GSList **pilots;
@@ -533,21 +630,21 @@ gpilotd_corba_monitor_on(GNOME_Pilot_Daemon obj,
 
 	GET_AND_CHECK_CLIENT_ID_VOID(client_id,cb);
 
-	LOG("monitor_on(pilot_id=\"%s\",client_id = %.20s...)",pilot_id,client_id);
+	LOG("monitor_on(pilot_name=\"%s\",client_id = %.20s...)",pilot_name,client_id);
 
-	if(pilot_id_from_name(pilot_id,orb_context->gpilotd_context)==0) {
+	if(pilot_id_from_name(pilot_name,orb_context->gpilotd_context)==0) {
 		g_free(client_id);
-		gpthrow_no_val_with_settings(UnknownPilot,exn,exn->pilotId = CORBA_string_dup(pilot_id));
+		gpthrow_no_val_with_settings(UnknownPilot,exn,exn->pilotId = CORBA_string_dup(pilot_name));
 	}
 
 	if((pilots = g_hash_table_lookup(orb_context->monitors,client_id))==NULL) {
 		pilots = g_new0(GSList*,1);
-		*pilots = g_slist_prepend(*pilots,g_strdup(pilot_id));
+		*pilots = g_slist_prepend(*pilots,g_strdup(pilot_name));
 		g_hash_table_insert(orb_context->monitors,g_strdup(client_id),pilots);
 	} else {
-		element = g_slist_find_custom(*pilots,(gchar*)pilot_id,
+		element = g_slist_find_custom(*pilots,(gchar*)pilot_name,
 					      (GCompareFunc)strcmp);
-		*pilots = g_slist_prepend(*pilots,g_strdup(pilot_id));
+		*pilots = g_slist_prepend(*pilots,g_strdup(pilot_name));
 	}
 
 	g_free(client_id);
@@ -699,19 +796,19 @@ monitor_off_helper(const gchar *pilot,
 static void 
 gpilotd_corba_monitor_off(GNOME_Pilot_Daemon obj,
 			  const GNOME_Pilot_Client cb, 
-			  const CORBA_char *pilot_id, 
+			  const CORBA_char *pilot_name, 
 			  CORBA_Environment *ev) 
 {
 	gchar *client_id;
 
 	GET_AND_CHECK_CLIENT_ID_VOID(client_id,cb);
 
-	LOG("monitor_off(pilot_id=\"%.10s\", client_id = %.20s...)",pilot_id,client_id);
-	if(pilot_id_from_name(pilot_id,orb_context->gpilotd_context)==0) {
+	LOG("monitor_off(pilot_name=\"%.10s\", client_id = %.20s...)",pilot_name,client_id);
+	if(pilot_id_from_name(pilot_name,orb_context->gpilotd_context)==0) {
 		g_free(client_id);
-		gpthrow_no_val_with_settings(UnknownPilot,exn,exn->pilotId = CORBA_string_dup(pilot_id));
+		gpthrow_no_val_with_settings(UnknownPilot,exn,exn->pilotId = CORBA_string_dup(pilot_name));
 	}
-	monitor_off_helper(pilot_id,client_id);
+	monitor_off_helper(pilot_name,client_id);
 
 	g_free(client_id);
 }
@@ -820,6 +917,58 @@ gpilotd_corba_notify_on(GNOME_Pilot_Daemon obj,
 	g_free(client_id);
 };
 
+static void
+gpilotd_corba_notify_off(GNOME_Pilot_Daemon obj, 
+			 GNOME_Pilot_EventType event, 
+			 const GNOME_Pilot_Client cb, 
+			 CORBA_Environment * ev) 
+{
+	GSList **pilots;
+	GPilotd_notify_on_helper_foreach_carrier carrier;
+	gchar *client_id;
+
+	GET_AND_CHECK_CLIENT_ID_VOID(client_id,cb);
+
+	{
+		gchar *event_type;
+		switch(event) {
+		case GNOME_Pilot_NOTIFY_CONNECT: 
+			event_type = g_strdup("CONNECT"); break;
+		case GNOME_Pilot_NOTIFY_DISCONNECT: 
+			event_type = g_strdup("DISCONNECT"); break;
+		case GNOME_Pilot_NOTIFY_BACKUP: 
+			event_type = g_strdup("BACKUP"); break;
+		case GNOME_Pilot_NOTIFY_CONDUIT: 
+			event_type = g_strdup("CONDUIT"); break;
+		case GNOME_Pilot_NOTIFY_REQUEST_COMPLETION: 
+			event_type = g_strdup("REQUEST_COMPLETION"); break;
+		case GNOME_Pilot_NOTIFY_REQUEST_TIMEOUT: 
+			event_type = g_strdup("REQUEST_TIMEOUT"); break;
+		case GNOME_Pilot_NOTIFY_USERINFO_REQUESTED: 
+			event_type = g_strdup("USERINFO_REQUESTED"); break;
+		case GNOME_Pilot_NOTIFY_USERINFO_SENT: 
+			event_type = g_strdup("USERINFO_SENT"); break;
+		default: g_assert_not_reached(); break;
+		}
+		LOG("corba: notify_off(event_type=%s,callback=%.20s...)",
+		    event_type,client_id);
+		g_free(event_type);
+	}
+
+	if((pilots = g_hash_table_lookup(orb_context->monitors,client_id))==NULL) {
+		g_free(client_id);
+		gpthrow_no_val(NoMonitors);
+	}
+
+	carrier.client_id = client_id;
+	carrier.event = event;
+	g_warning("Unimplemented method");
+	/*
+	g_slist_foreach(*pilots,(GFunc)notify_on_helper,(gpointer)&carrier);
+	*/
+	g_free(client_id);
+};
+
 
 /* get_* methods */
 static GNOME_Pilot_StringSequence*
@@ -843,26 +992,6 @@ gpilotd_corba_get_users(GNOME_Pilot_Daemon obj,
 		return empty_StringSequence();
 }
 
-static GNOME_Pilot_StringSequence*
-gpilotd_corba_get_conduits(GNOME_Pilot_Daemon obj, 
-			   CORBA_Environment *ev) 
-{
-
-	LOG("corba: FIXME get_conduits(...)");
-
-	/* FIXME: not implemented yet */
-
-	return empty_StringSequence();
-	/*
-	  conduits = GNOME_Pilot_StringSequence__alloc();
-	  conduits->_length = 1;
-	  conduits->_buffer = CORBA_sequence_CORBA_string_allocbuf(conduits->_length);
-	  conduits->_buffer[0] = CORBA_string_alloc(10);
-	  strcpy(conduits->_buffer[0],"conduit");
-	  return conduits;
-	*/
-}
-
 static GNOME_Pilot_StringSequence*
 gpilotd_corba_get_cradles(GNOME_Pilot_Daemon obj, 
 			  CORBA_Environment *ev) 
@@ -1023,7 +1152,7 @@ gpilotd_corba_get_pilots_by_login(GNOME_Pilot_Daemon obj,
 
 static CORBA_char*
 gpilotd_corba_get_user_name_by_pilot(GNOME_Pilot_Daemon obj, 
-				     const CORBA_char * pilot_id, 
+				     const CORBA_char * pilot_name, 
 				     CORBA_Environment *ev) 
 {
 	LOG("corba: FIXME get_user_name_by_pilot");
@@ -1034,7 +1163,7 @@ gpilotd_corba_get_user_name_by_pilot(GNOME_Pilot_Daemon obj,
 
 static CORBA_char*
 gpilotd_corba_get_user_login_by_pilot(GNOME_Pilot_Daemon obj, 
-				      const CORBA_char * pilot_id, 
+				      const CORBA_char * pilot_name, 
 				      CORBA_Environment *ev) 
 {
 	LOG("corba: FIXME get_user_login_by_pilot");
@@ -1045,10 +1174,10 @@ gpilotd_corba_get_user_login_by_pilot(GNOME_Pilot_Daemon obj,
 
 static CORBA_char* 
 gpilotd_get_pilot_base_dir(PortableServer_Servant _servant, 
-		     CORBA_char * pilot_id, 
+		     CORBA_char * pilot_name, 
 		     CORBA_Environment * ev) {
 	int i;
-	LOG("corba: get_pilot_base_dir(id=%s:",pilot_id);
+	LOG("corba: get_pilot_base_dir(id=%s:",pilot_name);
 	
 
 	/* FIXME: this could probably be prettyfied with some _foreach thingies... */
@@ -1056,7 +1185,7 @@ gpilotd_get_pilot_base_dir(PortableServer_Servant _servant,
 		/* FIXME: this could probably be prettyfied with some _foreach thingies... */
 		for(i=0;i<g_list_length(orb_context->gpilotd_context->pilots);i++) {
 			if(strcmp(GPILOT_PILOT(g_list_nth(orb_context->gpilotd_context->pilots,i)->data)->name,
-				  pilot_id)==0) {
+				  pilot_name)==0) {
 				LOG("match on %s",GPILOT_PILOT(g_list_nth(orb_context->gpilotd_context->pilots,i)->data)->name);
 				if(GPILOT_PILOT(g_list_nth(orb_context->gpilotd_context->pilots,i)->data)->sync_options.basedir!=NULL) {
 					return CORBA_string_dup(GPILOT_PILOT(g_list_nth(orb_context->gpilotd_context->pilots,i)->data)->sync_options.basedir);
@@ -1065,7 +1194,7 @@ gpilotd_get_pilot_base_dir(PortableServer_Servant _servant,
 				}
 			}
 		}
-		LOG("no matches on %s",pilot_id);
+		LOG("no matches on %s",pilot_name);
 		return CORBA_string_dup("");
 	} else 
 		return CORBA_string_dup("");
@@ -1073,28 +1202,79 @@ gpilotd_get_pilot_base_dir(PortableServer_Servant _servant,
 
 static CORBA_long 
 gpilotd_get_pilot_id_from_name(PortableServer_Servant _servant, 
-			       CORBA_char * pilot_id, 
+			       CORBA_char * pilot_name, 
 			       CORBA_Environment * ev) {
 	LOG("corba: get_user_pilot_id_from_name");
 
-	return pilot_id_from_name(pilot_id,orb_context->gpilotd_context);
+	return pilot_id_from_name(pilot_name,orb_context->gpilotd_context);
 }
 
 static CORBA_char* 
 gpilotd_get_pilot_name_from_id(PortableServer_Servant _servant, 
-			       CORBA_long pilot_id, 
+			       CORBA_long pilot_name, 
 			       CORBA_Environment * ev) {
 	gchar *tmp;
 	CORBA_char *retval;
-	LOG("corba: get_pilot_name_from_id(id=%d)",pilot_id);
+	LOG("corba: get_pilot_name_from_id(id=%d)",pilot_name);
 
-	tmp = pilot_name_from_id(pilot_id,orb_context->gpilotd_context);
+	tmp = pilot_name_from_id(pilot_name,orb_context->gpilotd_context);
 	retval =  CORBA_string_dup(tmp);
 	g_free(tmp);
 
 	return retval;
 }
 
+static GNOME_Pilot_StringSequence*
+gpilotd_corba_get_databases_from_cache(GNOME_Pilot_Daemon obj, 
+				       const CORBA_char *pilot_name,
+				       CORBA_Environment *ev) 
+{
+	char **databases,*pfx;
+	int num_bases;
+	guint32 pilot_id;
+	LOG("corba: get_databases_from_cache(...)");
+
+	pilot_id = pilot_id_from_name(pilot_name,
+				      orb_context->gpilotd_context);
+
+	/* If no pilot matches the name, set the exn, and exit with a empty stringsequence */
+	if (pilot_id == 0) {
+		/* gpthrow_no_val_with_settings(UnknownPilot,exn,exn->pilotId = CORBA_string_dup(pilot_name)); */
+		return empty_StringSequence();
+		return NULL;
+	} else {
+		pfx = g_strdup_printf("/gnome-pilot.d/PilotCache%d/Databases/",
+				      pilot_id);
+		gnome_config_push_prefix(pfx);
+		gnome_config_get_vector("databases",&num_bases,&databases);	
+		gnome_config_pop_prefix();
+		
+		/* if there were databases, copy into a corba array
+                   and return, otherwise return an empty array */
+		if(num_bases > 0) {
+			GNOME_Pilot_StringSequence *pilots;
+			int i;
+			
+			pilots = GNOME_Pilot_StringSequence__alloc();
+			pilots->_length = num_bases;
+			pilots->_buffer = CORBA_sequence_CORBA_string_allocbuf(pilots->_length);
+			
+			/* FIXME: this could probably be prettyfied
+                           with some _foreach thingies... */
+			for(i = 0; i < num_bases; i++) {
+				pilots->_buffer[i] = CORBA_string_alloc(strlen(databases[i]));
+				strcpy(pilots->_buffer[i],databases[i]);
+				g_free(databases[i]);
+			}
+			g_free(databases);
+			return pilots;
+		} else {
+			return empty_StringSequence();
+		}
+	}
+}
+
+
 
 /*************************************************************************************/
 static gint 
@@ -1125,7 +1305,6 @@ pilot_id_from_name(const gchar *name,
 				   (GCompareFunc)match_pilot_and_name);
 	if(pilot)
 		return ((GPilotPilot*)pilot->data)->pilot_id;
-	g_warning("unknown pilot name \"%s\"",name);
 	return 0;
 }
 
@@ -1140,7 +1319,6 @@ pilot_name_from_id(guint32 id,
 	if(pilot)
 		return g_strdup(((GPilotPilot*)pilot->data)->name);
 
-	g_warning("unknown pilot id %d",id);
 	return NULL;
 }
 
@@ -1156,13 +1334,14 @@ PortableServer_ServantBase__epv base_epv = {
 
 POA_GNOME_Pilot_Daemon__epv daemon_epv = { 
 	NULL, 
+	(gpointer)&gpilotd_corba_pause,
 	(gpointer)&gpilotd_corba_request_install,
 	(gpointer)&gpilotd_corba_request_restore,
+	(gpointer)&gpilotd_corba_request_revive,
 	(gpointer)&gpilotd_corba_request_conduit,
 	(gpointer)&gpilotd_corba_request_remove,
 	(gpointer)&gpilotd_corba_get_system_info,
 	(gpointer)&gpilotd_corba_get_users,
-	(gpointer)&gpilotd_corba_get_conduits,
 	(gpointer)&gpilotd_corba_get_cradles,
 	(gpointer)&gpilotd_corba_get_pilots,
 	(gpointer)&gpilotd_corba_get_pilot_ids,
@@ -1173,12 +1352,13 @@ POA_GNOME_Pilot_Daemon__epv daemon_epv = {
 	(gpointer)&gpilotd_get_pilot_base_dir,
 	(gpointer)&gpilotd_get_pilot_id_from_name,
 	(gpointer)&gpilotd_get_pilot_name_from_id,
+	(gpointer)&gpilotd_corba_get_databases_from_cache,
 	(gpointer)&gpilotd_corba_get_user_info,
 	(gpointer)&gpilotd_corba_set_user_info,
-	/*(gpointer)&gpilotd_corba_set_user_info_if_not_set,*/
 	(gpointer)&gpilotd_corba_monitor_on,
 	(gpointer)&gpilotd_corba_monitor_off,
 	(gpointer)&gpilotd_corba_notify_on,
+	(gpointer)&gpilotd_corba_notify_off,
 };
 
 POA_GNOME_Pilot_Daemon__vepv daemon_vepv = { &base_epv, &daemon_epv };
@@ -1778,7 +1958,7 @@ void gpilotd_corba_init(int *argc,
 	switch(goad_server_register(CORBA_OBJECT_NIL,acc,"gpilotd","object",&ev)) {
 	case -2:
 		/* FIXME: better handling here */
-		g_print("Already running\n");
+		g_message(_("The gnome-pilot daemon is already running"));
 		exit (0);
 		break;
 	case -1:
