@@ -66,6 +66,7 @@ static GPilotRequestType request_type_from_string(gchar *str) {
 
 	if(g_strcasecmp(str,"GREQ_INSTALL")==0) return GREQ_INSTALL;
 	if(g_strcasecmp(str,"GREQ_RESTORE")==0) return GREQ_RESTORE;
+	if(g_strcasecmp(str,"GREQ_REVIVE")==0) return GREQ_REVIVE;
 	if(g_strcasecmp(str,"GREQ_CONDUIT")==0) return GREQ_CONDUIT;
 	if(g_strcasecmp(str,"GREQ_SET_USERINFO")==0) return GREQ_SET_USERINFO;
 	if(g_strcasecmp(str,"GREQ_GET_USERINFO")==0) return GREQ_GET_USERINFO;
@@ -173,6 +174,12 @@ GPilotRequest* gpc_queue_load_request(guint32 pilot_id,gboolean _type,guint num)
 	case GREQ_RESTORE:
 		req->parameters.restore.directory = gnome_config_get_string(ENT_DIRECTORY);
 		break;
+	case GREQ_REVIVE:
+		req->parameters.set_userinfo.password = gnome_config_get_string(ENT_PASSWORD);
+		req->parameters.set_userinfo.user_id = gnome_config_get_string(ENT_USER_ID);
+		req->parameters.set_userinfo.pilot_id = gnome_config_get_int(ENT_PILOT_ID);
+		req->parameters.revive.directory = gnome_config_get_string(ENT_DIRECTORY);
+		break;
 	case GREQ_CONDUIT: {
 		gchar *tmp;
 		req->parameters.conduit.name = gnome_config_get_string(ENT_CONDUIT);
@@ -236,12 +243,20 @@ guint gpc_queue_store_request(GPilotRequest req) {
 		gnome_config_set_string(ENT_TYPE,"GREQ_RESTORE"); 
 		gnome_config_set_string(ENT_DIRECTORY,req.parameters.restore.directory);
 		break;
+	case GREQ_REVIVE:
+		gnome_config_set_string(ENT_TYPE,"GREQ_REVIVE"); 
+		gnome_config_set_string(ENT_DEVICE,req.cradle);
+		gnome_config_set_string(ENT_DIRECTORY,req.parameters.restore.directory);
+		gnome_config_set_string(ENT_PASSWORD,req.parameters.set_userinfo.password);
+		gnome_config_set_string(ENT_USER_ID,req.parameters.set_userinfo.user_id);
+		gnome_config_set_int(ENT_PILOT_ID,req.parameters.set_userinfo.pilot_id);
+		break;
 	case GREQ_CONDUIT: 
 		g_message("req.parameters.conduit.name = %s",req.parameters.conduit.name);
 		gnome_config_set_string(ENT_TYPE,"GREQ_CONDUIT"); 
 		gnome_config_set_string(ENT_CONDUIT,req.parameters.conduit.name);
 		gnome_config_set_string(ENT_HOW,
-					gnome_pilot_conduit_sync_type_int_to_str(req.parameters.conduit.how));
+		gnome_pilot_conduit_sync_type_int_to_str(req.parameters.conduit.how));
 		break;
 	case GREQ_NEW_USERINFO: 
 		gnome_config_set_string(ENT_TYPE,"GREQ_NEW_USERINFO"); 
@@ -252,6 +267,7 @@ guint gpc_queue_store_request(GPilotRequest req) {
 	case GREQ_SET_USERINFO: 
 		gnome_config_set_string(ENT_TYPE,"GREQ_SET_USERINFO"); 
 		gnome_config_set_string(ENT_DEVICE,req.cradle);
+		gnome_config_set_string(ENT_PASSWORD,req.parameters.set_userinfo.password);
 		gnome_config_set_string(ENT_USER_ID,req.parameters.set_userinfo.user_id);
 		gnome_config_set_int(ENT_PILOT_ID,req.parameters.set_userinfo.pilot_id);
 		break;
@@ -318,6 +334,11 @@ gpc_queue_purge_request(GPilotRequest **req)
 	case GREQ_RESTORE:
 		g_free((*req)->parameters.restore.directory);
 		break;
+	case GREQ_REVIVE:
+		g_free((*req)->parameters.restore.directory);
+		g_free((*req)->parameters.set_userinfo.user_id);
+		g_free((*req)->parameters.set_userinfo.password);
+		break;
 	case GREQ_CONDUIT:
 		g_free((*req)->parameters.conduit.name);
 		break;
