@@ -29,6 +29,7 @@ typedef enum {
   GREQ_GET_USERINFO,
   GREQ_SET_USERINFO,
   GREQ_NEW_USERINFO,
+  GREQ_REVIVE,
 
   GREQ_INVALID=-1,
 } GPilotRequestType;
@@ -50,6 +51,12 @@ typedef struct GPilotRequest {
 		struct {
 			gchar *directory;
 		} restore;
+		struct {
+			gchar *password;
+			gchar *user_id;
+			guint32 pilot_id;
+			gchar *directory;
+		} revive;
 		struct {
 			gchar *name;
 			GnomePilotConduitSyncType how;
